@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtQuick module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qquickparticlesystem_p.h"
 #include <QtQuick/qsgnode.h>
@@ -324,7 +288,7 @@ QQuickParticleGroupData::QQuickParticleGroupData(const QString &name, QQuickPart
 
 QQuickParticleGroupData::~QQuickParticleGroupData()
 {
-    for (QQuickParticleData *d : qAsConst(data))
+    for (QQuickParticleData *d : std::as_const(data))
         delete d;
 }
 
@@ -347,7 +311,7 @@ void QQuickParticleGroupData::setSize(int newSize)
     }
     int delta = newSize - m_size;
     m_size = newSize;
-    for (QQuickParticlePainter *p : qAsConst(painters))
+    for (QQuickParticlePainter *p : std::as_const(painters))
         p->setCount(p->count() + delta);
 }
 
@@ -360,7 +324,7 @@ void QQuickParticleGroupData::kill(QQuickParticleData* d)
 {
     Q_ASSERT(d->groupId == index);
     d->lifeSpan = 0;//Kill off
-    for (QQuickParticlePainter *p : qAsConst(painters))
+    for (QQuickParticlePainter *p : std::as_const(painters))
         p->reload(d);
     freeList.free(d->index);
 }
@@ -579,7 +543,7 @@ QQuickParticleSystem::QQuickParticleSystem(QQuickItem *parent) :
 
 QQuickParticleSystem::~QQuickParticleSystem()
 {
-    for (QQuickParticleGroupData *gd : qAsConst(groupData))
+    for (QQuickParticleGroupData *gd : std::as_const(groupData))
         delete gd;
 }
 
@@ -593,10 +557,10 @@ void QQuickParticleSystem::initGroups()
     groupIds.clear();
     nextFreeGroupId = 0;
 
-    for (auto e : qAsConst(m_emitters)) {
+    for (auto e : std::as_const(m_emitters)) {
         e->reclaculateGroupId();
     }
-    for (QQuickParticlePainter *p : qAsConst(m_painters)) {
+    for (QQuickParticlePainter *p : std::as_const(m_painters)) {
         p->recalculateGroupIds();
     }
 
@@ -668,7 +632,7 @@ void QQuickParticleSystem::setPaused(bool arg) {
         if (m_animation && m_animation->state() != QAbstractAnimation::Stopped)
             m_paused ? m_animation->pause() : m_animation->resume();
         if (!m_paused) {
-            for (QQuickParticlePainter *p : qAsConst(m_painters)) {
+            for (QQuickParticlePainter *p : std::as_const(m_painters)) {
                 if (p) {
                     p->update();
                 }
@@ -777,12 +741,12 @@ void QQuickParticleSystem::reset()
     if (!m_running)
         return;
 
-    for (QQuickParticleEmitter *e : qAsConst(m_emitters))
+    for (QQuickParticleEmitter *e : std::as_const(m_emitters))
         e->reset();
 
     emittersChanged();
 
-    for (QQuickParticlePainter *p : qAsConst(m_painters)) {
+    for (QQuickParticlePainter *p : std::as_const(m_painters)) {
         loadPainter(p);
         p->reset();
     }
@@ -843,7 +807,7 @@ void QQuickParticleSystem::emittersChanged()
     }
 
     // Populate groups and set sizes.
-    for (int i = 0; i < m_emitters.count(); ) {
+    for (int i = 0; i < m_emitters.size(); ) {
         QQuickParticleEmitter *e = m_emitters.at(i);
         if (!e) {
             m_emitters.removeAt(i);
@@ -875,13 +839,13 @@ void QQuickParticleSystem::emittersChanged()
     if (particleCount > bySysIdx.size())//New datum requests haven't updated it
         bySysIdx.resize(particleCount);
 
-    for (QQuickParticleAffector *a : qAsConst(m_affectors)) {//Groups may have changed
+    for (QQuickParticleAffector *a : std::as_const(m_affectors)) {//Groups may have changed
         if (a) {
             a->m_updateIntSet = true;
         }
     }
 
-    for (QQuickParticlePainter *p : qAsConst(m_painters))
+    for (QQuickParticlePainter *p : std::as_const(m_painters))
         loadPainter(p);
 
     if (!m_groups.isEmpty())
@@ -896,7 +860,7 @@ void QQuickParticleSystem::createEngine()
     if (stateEngine && m_debugMode)
         qDebug() << "Resetting Existing Sprite Engine...";
     //### Solve the losses if size/states go down
-    for (QQuickParticleGroup *group : qAsConst(m_groups)) {
+    for (QQuickParticleGroup *group : std::as_const(m_groups)) {
         bool exists = false;
         for (auto it = groupIds.keyBegin(), end = groupIds.keyEnd(); it != end; ++it) {
             if (group->name() == *it) {
@@ -909,14 +873,14 @@ void QQuickParticleSystem::createEngine()
         }
     }
 
-    if (m_groups.count()) {
+    if (m_groups.size()) {
         //Reorder groups List so as to have the same order as groupData
         // TODO: can't we just merge the two lists?
         QList<QQuickParticleGroup*> newList;
         for (int i = 0, ei = groupData.size(); i != ei; ++i) {
             bool exists = false;
             QString name = groupData[i]->name();
-            for (QQuickParticleGroup *existing : qAsConst(m_groups)) {
+            for (QQuickParticleGroup *existing : std::as_const(m_groups)) {
                 if (existing->name() == name) {
                     newList << existing;
                     exists = true;
@@ -929,8 +893,8 @@ void QQuickParticleSystem::createEngine()
         }
         m_groups = newList;
         QList<QQuickStochasticState*> states;
-        states.reserve(m_groups.count());
-        for (QQuickParticleGroup *g : qAsConst(m_groups))
+        states.reserve(m_groups.size());
+        for (QQuickParticleGroup *g : std::as_const(m_groups))
             states << (QQuickStochasticState*)g;
 
         if (!stateEngine)
@@ -988,7 +952,7 @@ int QQuickParticleSystem::nextSystemIndex()
 
 QQuickParticleData* QQuickParticleSystem::newDatum(int groupId, bool respectLimits, int sysIndex)
 {
-    Q_ASSERT(groupId < groupData.count());//XXX shouldn't really be an assert
+    Q_ASSERT(groupId < groupData.size());//XXX shouldn't really be an assert
 
     QQuickParticleData* ret = groupData[groupId]->newDatum(respectLimits);
     if (!ret) {
@@ -1035,10 +999,10 @@ void QQuickParticleSystem::finishNewDatum(QQuickParticleData *pd)
     Q_ASSERT(pd);
     groupData[pd->groupId]->prepareRecycler(pd);
 
-    for (QQuickParticleAffector *a : qAsConst(m_affectors))
+    for (QQuickParticleAffector *a : std::as_const(m_affectors))
         if (a && a->m_needsReset)
             a->reset(pd);
-    for (QQuickParticlePainter *p : qAsConst(groupData[pd->groupId]->painters))
+    for (QQuickParticlePainter *p : std::as_const(groupData[pd->groupId]->painters))
         if (p)
             p->load(pd);
 }
@@ -1061,18 +1025,18 @@ void QQuickParticleSystem::updateCurrentTime( int currentTime )
 
     bool oldClear = m_empty;
     m_empty = true;
-    for (QQuickParticleGroupData *gd : qAsConst(groupData))//Recycle all groups and see if they're out of live particles
+    for (QQuickParticleGroupData *gd : std::as_const(groupData))//Recycle all groups and see if they're out of live particles
         m_empty = gd->recycle() && m_empty;
 
     if (stateEngine)
         stateEngine->updateSprites(timeInt);
 
-    for (QQuickParticleEmitter *emitter : qAsConst(m_emitters))
+    for (QQuickParticleEmitter *emitter : std::as_const(m_emitters))
         emitter->emitWindow(timeInt);
-    for (QQuickParticleAffector *a : qAsConst(m_affectors))
+    for (QQuickParticleAffector *a : std::as_const(m_affectors))
         a->affectSystem(dt);
     for (QQuickParticleData *d : needsReset)
-        for (QQuickParticlePainter *p : qAsConst(groupData[d->groupId]->painters))
+        for (QQuickParticlePainter *p : std::as_const(groupData[d->groupId]->painters))
             p->reload(d);
 
     if (oldClear != m_empty)
