@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtQml module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qqmlnativedebugservice.h"
 
@@ -296,7 +260,7 @@ void NativeDebugger::signalEmitted(const QString &signal)
     //Normalize to Lower case.
     QString signalName = signal.left(signal.indexOf(QLatin1Char('('))).toLower();
 
-    for (const QString &signal : qAsConst(breakOnSignals)) {
+    for (const QString &signal : std::as_const(breakOnSignals)) {
         if (signal == signalName) {
             // TODO: pause debugger
             break;
@@ -746,7 +710,7 @@ void QQmlNativeDebugServiceImpl::engineAboutToBeRemoved(QJSEngine *engine)
 void QQmlNativeDebugServiceImpl::stateAboutToBeChanged(QQmlDebugService::State state)
 {
     if (state == Enabled) {
-        for (NativeDebugger *debugger : qAsConst(m_debuggers)) {
+        for (NativeDebugger *debugger : std::as_const(m_debuggers)) {
             QV4::ExecutionEngine *engine = debugger->engine();
             if (!engine->debugger())
                 engine->setDebugger(debugger);
@@ -770,7 +734,7 @@ void QQmlNativeDebugServiceImpl::messageReceived(const QByteArray &message)
     } else if (cmd == QLatin1String("echo")) {
         response.insert(QStringLiteral("result"), arguments);
     } else {
-        for (NativeDebugger *debugger : qAsConst(m_debuggers))
+        for (NativeDebugger *debugger : std::as_const(m_debuggers))
             if (debugger)
                 debugger->handleCommand(&response, cmd, arguments);
     }
