@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2020 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtQml module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2020 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include <QtQml/qjsmanagedvalue.h>
 #include <QtQml/qjsengine.h>
@@ -994,7 +958,7 @@ QJSValue QJSManagedValue::call(const QJSValueList &arguments) const
     QV4::ExecutionEngine *engine = f->engine();
 
     QV4::Scope scope(engine);
-    QV4::JSCallArguments jsCallData(scope, arguments.length());
+    QV4::JSCallArguments jsCallData(scope, arguments.size());
     *jsCallData.thisObject = engine->globalObject;
     int i = 0;
     for (const QJSValue &arg : arguments) {
@@ -1033,7 +997,7 @@ QJSValue QJSManagedValue::callWithInstance(const QJSValue &instance,
     }
 
     QV4::Scope scope(engine);
-    QV4::JSCallArguments jsCallData(scope, arguments.length());
+    QV4::JSCallArguments jsCallData(scope, arguments.size());
     *jsCallData.thisObject = QJSValuePrivate::convertToReturnedValue(engine, instance);
     int i = 0;
     for (const QJSValue &arg : arguments) {
@@ -1066,7 +1030,7 @@ QJSValue QJSManagedValue::callAsConstructor(const QJSValueList &arguments) const
     QV4::ExecutionEngine *engine = f->engine();
 
     QV4::Scope scope(engine);
-    QV4::JSCallArguments jsCallData(scope, arguments.length());
+    QV4::JSCallArguments jsCallData(scope, arguments.size());
     int i = 0;
     for (const QJSValue &arg : arguments) {
         if (Q_UNLIKELY(!QJSValuePrivate::checkEngine(engine, arg))) {
