@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtQml module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include <QtCore/qstring.h>
 #include <QtCore/qvarlengtharray.h>
@@ -142,6 +106,16 @@
       integers.append(jsArray.property(i).toInt());
   }
   \endcode
+
+  \section2 Converting to JSON
+
+  It's possible to convert a QJSValue to a JSON type. For example,
+  to convert to an array, use \l QJSEngine::fromScriptValue():
+
+  \code
+  const QJsonValue jsonValue = engine.fromScriptValue<QJsonValue>(jsValue);
+  const QJsonArray jsonArray = jsonValue.toArray();
+  \endcode
 */
 
 /*!
@@ -526,7 +500,7 @@ double QJSValue::toNumber() const
 bool QJSValue::toBool() const
 {
     if (const QString *string = QJSValuePrivate::asQString(this))
-        return string->length() > 0;
+        return string->size() > 0;
 
     return caughtResult<bool>(this, &QV4::Value::toBoolean);
 }
@@ -700,7 +674,7 @@ QJSValue QJSValue::call(const QJSValueList &args) const
     Q_ASSERT(engine);
 
     Scope scope(engine);
-    JSCallArguments jsCallData(scope, args.length());
+    JSCallArguments jsCallData(scope, args.size());
     *jsCallData.thisObject = engine->globalObject;
     for (int i = 0; i < args.size(); ++i) {
         if (!QJSValuePrivate::checkEngine(engine, args.at(i))) {
@@ -713,7 +687,7 @@ QJSValue QJSValue::call(const QJSValueList &args) const
     ScopedValue result(scope, f->call(jsCallData));
     if (engine->hasException)
         result = engine->catchException();
-    if (engine->isInterrupted.loadAcquire())
+    if (engine->isInterrupted.loadRelaxed())
         result = engine->newErrorObject(QStringLiteral("Interrupted"));
 
     return QJSValuePrivate::fromReturnedValue(result->asReturnedValue());
@@ -767,7 +741,7 @@ QJSValue QJSValue::callWithInstance(const QJSValue &instance, const QJSValueList
     ScopedValue result(scope, f->call(jsCallData));
     if (engine->hasException)
         result = engine->catchException();
-    if (engine->isInterrupted.loadAcquire())
+    if (engine->isInterrupted.loadRelaxed())
         result = engine->newErrorObject(QStringLiteral("Interrupted"));
 
     return QJSValuePrivate::fromReturnedValue(result->asReturnedValue());
@@ -813,7 +787,7 @@ QJSValue QJSValue::callAsConstructor(const QJSValueList &args) const
     ScopedValue result(scope, f->callAsConstructor(jsCallData));
     if (engine->hasException)
         result = engine->catchException();
-    if (engine->isInterrupted.loadAcquire())
+    if (engine->isInterrupted.loadRelaxed())
         result = engine->newErrorObject(QStringLiteral("Interrupted"));
 
     return QJSValuePrivate::fromReturnedValue(result->asReturnedValue());
