@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtQml module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qqmlproperty.h"
 #include "qqmlproperty_p.h"
@@ -56,6 +20,9 @@
 #include "qqmlvaluetypeproxybinding_p.h"
 #include <private/qjsvalue_p.h>
 #include <private/qv4functionobject_p.h>
+#include <private/qv4qobjectwrapper_p.h>
+#include <private/qqmlbuiltinfunctions_p.h>
+#include <private/qqmlirbuilder_p.h>
 
 #include <QStringList>
 #include <QVector>
@@ -120,10 +87,7 @@ qWarning() << "Pixel size should now be 24:" << property.read().toInt();
 /*!
     Create an invalid QQmlProperty.
 */
-QQmlProperty::QQmlProperty()
-: d(nullptr)
-{
-}
+QQmlProperty::QQmlProperty() = default;
 
 /*!  \internal */
 QQmlProperty::~QQmlProperty()
@@ -152,8 +116,10 @@ QQmlProperty::QQmlProperty(QObject *obj)
 QQmlProperty::QQmlProperty(QObject *obj, QQmlContext *ctxt)
 : d(new QQmlPropertyPrivate)
 {
-    d->context = ctxt?QQmlContextData::get(ctxt):nullptr;
-    d->engine = ctxt?ctxt->engine():nullptr;
+    if (ctxt) {
+        d->context = QQmlContextData::get(ctxt);
+        d->engine = ctxt->engine();
+    }
     d->initDefault(obj);
 }
 
@@ -204,12 +170,15 @@ QQmlProperty::QQmlProperty(QObject *obj, const QString &name)
 QQmlProperty::QQmlProperty(QObject *obj, const QString &name, QQmlContext *ctxt)
 : d(new QQmlPropertyPrivate)
 {
-    d->context = ctxt?QQmlContextData::get(ctxt):nullptr;
-    d->engine = ctxt?ctxt->engine():nullptr;
+    if (ctxt) {
+        d->context = QQmlContextData::get(ctxt);
+        d->engine = ctxt->engine();
+    }
+
     d->initProperty(obj, name);
     if (!isValid()) {
         d->object = nullptr;
-        d->context = nullptr;
+        d->context.reset();
         d->engine = nullptr;
     }
 }
@@ -226,23 +195,24 @@ QQmlProperty::QQmlProperty(QObject *obj, const QString &name, QQmlEngine *engine
     d->initProperty(obj, name);
     if (!isValid()) {
         d->object = nullptr;
-        d->context = nullptr;
+        d->context.reset();
         d->engine = nullptr;
     }
 }
 
 QQmlProperty QQmlPropertyPrivate::create(QObject *target, const QString &propertyName,
-                                         const QQmlRefPointer<QQmlContextData> &context)
+                                         const QQmlRefPointer<QQmlContextData> &context,
+                                         QQmlPropertyPrivate::InitFlags flags)
 {
     QQmlProperty result;
     auto d = new QQmlPropertyPrivate;
     result.d = d;
     d->context = context;
     d->engine = context ? context->engine() : nullptr;
-    d->initProperty(target, propertyName);
+    d->initProperty(target, propertyName, flags);
     if (!result.isValid()) {
         d->object = nullptr;
-        d->context = nullptr;
+        d->context.reset();
         d->engine = nullptr;
     }
     return result;
@@ -263,10 +233,10 @@ QQmlRefPointer<QQmlContextData> QQmlPropertyPrivate::effectiveContext() const
         return nullptr;
 }
 
-void QQmlPropertyPrivate::initProperty(QObject *obj, const QString &name)
+// ### Qt7: Do not accept the "onFoo" syntax for signals anymore, and change the flags accordingly.
+void QQmlPropertyPrivate::initProperty(QObject *obj, const QString &name,
+                                       QQmlPropertyPrivate::InitFlags flags)
 {
-    if (!obj) return;
-
     QQmlRefPointer<QQmlTypeNameCache> typeNameCache = context ? context->imports() : nullptr;
 
     QObject *currentObject = obj;
@@ -278,7 +248,7 @@ void QQmlPropertyPrivate::initProperty(QObject *obj, const QString &name)
         if (path.isEmpty()) return;
 
         // Everything up to the last property must be an "object type" property
-        for (int ii = 0; ii < path.count() - 1; ++ii) {
+        for (int ii = 0; ii < path.size() - 1; ++ii) {
             const QStringView &pathName = path.at(ii);
 
             // Types must begin with an uppercase letter (see checkRegistration()
@@ -294,7 +264,7 @@ void QQmlPropertyPrivate::initProperty(QObject *obj, const QString &name)
                         currentObject = qmlAttachedPropertiesObject(currentObject, func);
                         if (!currentObject) return; // Something is broken with the attachable type
                     } else if (r.importNamespace) {
-                        if (++ii == path.count())
+                        if (++ii == path.size())
                             return; // No type following the namespace
 
                         // TODO: Do we really _not_ want to query the namespaced types here?
@@ -324,14 +294,33 @@ void QQmlPropertyPrivate::initProperty(QObject *obj, const QString &name)
             }
 
             QQmlPropertyData local;
-            QQmlPropertyData *property =
-                    QQmlPropertyCache::property(engine, currentObject, pathName, context, &local);
+            const QQmlPropertyData *property = currentObject
+                    ? QQmlPropertyCache::property(currentObject, pathName, context, &local)
+                    : nullptr;
 
-            if (!property) return; // Not a property
-            if (property->isFunction())
+            if (!property) {
+                // Not a property; Might be an ID
+                // You can't look up an ID on a non-null object, though.
+                if (currentObject || !(flags & InitFlag::AllowId))
+                    return;
+
+                for (auto idContext = context; idContext; idContext = idContext->parent()) {
+                    const int objectId = idContext->propertyIndex(pathName.toString());
+                    if (objectId != -1 && objectId < idContext->numIdValues()) {
+                        currentObject = context->idValue(objectId);
+                        break;
+                    }
+                }
+
+                if (!currentObject)
+                    return;
+
+                continue;
+            } else if (property->isFunction()) {
                 return; // Not an object property
+            }
 
-            if (ii == (path.count() - 2) && QQmlMetaType::isValueType(property->propType())) {
+            if (ii == (path.size() - 2) && QQmlMetaType::isValueType(property->propType())) {
                 // We're now at a value type property
                 const QMetaObject *valueTypeMetaObject = QQmlMetaType::metaObjectForValueType(property->propType());
                 if (!valueTypeMetaObject) return; // Not a value type
@@ -367,26 +356,56 @@ void QQmlPropertyPrivate::initProperty(QObject *obj, const QString &name)
         }
 
         terminal = path.last();
+    } else if (!currentObject) {
+        return;
     }
 
-    if (terminal.size() >= 3 && terminal.at(0) == u'o' && terminal.at(1) == u'n'
-            && (terminal.at(2).isUpper() || terminal.at(2) == u'_')) {
+    auto findSignalInMetaObject = [&](const QByteArray &signalName) {
+        const QMetaMethod method = findSignalByName(currentObject->metaObject(), signalName);
+        if (!method.isValid())
+            return false;
 
-        QString signalName = terminal.mid(2).toString();
+        object = currentObject;
+        core.load(method);
+        return true;
+    };
+
+    QQmlData *ddata = QQmlData::get(currentObject, false);
+    auto findChangeSignal = [&](QStringView signalName) {
+        const QString changed = QStringLiteral("Changed");
+        if (signalName.endsWith(changed)) {
+            const QStringView propName = signalName.first(signalName.size() - changed.size());
+            const QQmlPropertyData *d = ddata->propertyCache->property(propName, currentObject, context);
+            while (d && d->isFunction())
+                d = ddata->propertyCache->overrideData(d);
+
+            if (d && d->notifyIndex() != -1) {
+                object = currentObject;
+                core = *ddata->propertyCache->signal(d->notifyIndex());
+                return true;
+            }
+        }
+        return false;
+    };
+
+    const QString terminalString = terminal.toString();
+    if (QmlIR::IRBuilder::isSignalPropertyName(terminalString)) {
+        QString signalName = terminalString.mid(2);
         int firstNon_;
-        int length = signalName.length();
+        int length = signalName.size();
         for (firstNon_ = 0; firstNon_ < length; ++firstNon_)
             if (signalName.at(firstNon_) != u'_')
                 break;
         signalName[firstNon_] = signalName.at(firstNon_).toLower();
 
-        // XXX - this code treats methods as signals
-
-        QQmlData *ddata = QQmlData::get(currentObject, false);
         if (ddata && ddata->propertyCache) {
-
             // Try method
-            QQmlPropertyData *d = ddata->propertyCache->property(signalName, currentObject, context);
+            const QQmlPropertyData *d = ddata->propertyCache->property(
+                        signalName, currentObject, context);
+
+            // ### Qt7: This code treats methods as signals. It should use d->isSignal().
+            //          That would be a change in behavior, though. Right now you can construct a
+            //          QQmlProperty from such a thing.
             while (d && !d->isFunction())
                 d = ddata->propertyCache->overrideData(d);
 
@@ -396,39 +415,53 @@ void QQmlPropertyPrivate::initProperty(QObject *obj, const QString &name)
                 return;
             }
 
-            // Try property
-            if (signalName.endsWith(QLatin1String("Changed"))) {
-                const QStringView propName = QStringView{signalName}.mid(0, signalName.length() - 7);
-                QQmlPropertyData *d = ddata->propertyCache->property(propName, currentObject, context);
-                while (d && d->isFunction())
-                    d = ddata->propertyCache->overrideData(d);
-
-                if (d && d->notifyIndex() != -1) {
-                    object = currentObject;
-                    core = *ddata->propertyCache->signal(d->notifyIndex());
-                    return;
-                }
-            }
-
-        } else {
-            QMetaMethod method = findSignalByName(currentObject->metaObject(),
-                                                  signalName.toLatin1());
-            if (method.isValid()) {
-                object = currentObject;
-                core.load(method);
+            if (findChangeSignal(signalName))
                 return;
-            }
+        } else if (findSignalInMetaObject(signalName.toUtf8())) {
+            return;
         }
     }
 
-    // Property
-    QQmlPropertyData local;
-    QQmlPropertyData *property =
-        QQmlPropertyCache::property(engine, currentObject, terminal, context, &local);
-    if (property && !property->isFunction()) {
-        object = currentObject;
-        core = *property;
-        nameCache = terminal.toString();
+    if (ddata && ddata->propertyCache) {
+        const QQmlPropertyData *property = ddata->propertyCache->property(
+                    terminal, currentObject, context);
+
+        // Technically, we might find an override that is not a function.
+        while (property && !property->isSignal()) {
+            if (!property->isFunction()) {
+                object = currentObject;
+                core = *property;
+                nameCache = terminalString;
+                return;
+            }
+            property = ddata->propertyCache->overrideData(property);
+        }
+
+        if (!(flags & InitFlag::AllowSignal))
+            return;
+
+        if (property) {
+            Q_ASSERT(property->isSignal());
+            object = currentObject;
+            core = *property;
+            return;
+        }
+
+        // At last: Try the change signal.
+        findChangeSignal(terminal);
+    } else {
+        // We might still find the property in the metaobject, even without property cache.
+        const QByteArray propertyName = terminal.toUtf8();
+        const QMetaProperty prop = findPropertyByName(currentObject->metaObject(), propertyName);
+
+        if (prop.isValid()) {
+            object = currentObject;
+            core.load(prop);
+            return;
+        }
+
+        if (flags & InitFlag::AllowSignal)
+            findSignalInMetaObject(terminal.toUtf8());
     }
 }
 
@@ -710,7 +743,6 @@ QString QQmlProperty::name() const
     if (!d)
         return QString();
     if (d->nameCache.isEmpty()) {
-        // ###
         if (!d->object) {
         } else if (d->isValueType()) {
             const QMetaObject *valueTypeMetaObject = QQmlMetaType::metaObjectForValueType(d->core.propType());
@@ -719,8 +751,15 @@ QString QQmlProperty::name() const
             const char *vtName = valueTypeMetaObject->property(d->valueTypeData.coreIndex()).name();
             d->nameCache = d->core.name(d->object) + QLatin1Char('.') + QString::fromUtf8(vtName);
         } else if (type() & SignalProperty) {
-            QString name = QLatin1String("on") + d->core.name(d->object);
-            name[2] = name.at(2).toUpper();
+            // ### Qt7: Return the original signal name here. Do not prepend "on"
+            QString name = QStringLiteral("on") + d->core.name(d->object);
+            for (int i = 2, end = name.size(); i != end; ++i) {
+                const QChar c = name.at(i);
+                if (c != u'_') {
+                    name[i] = c.toUpper();
+                    break;
+                }
+            }
             d->nameCache = name;
         } else {
             d->nameCache = d->core.name(d->object);
@@ -818,7 +857,7 @@ static void removeOldBinding(QObject *object, QQmlPropertyIndex index, QQmlPrope
     if (!oldBinding)
         return;
 
-    if (valueTypeIndex != -1 && oldBinding->isValueTypeProxy())
+    if (valueTypeIndex != -1 && oldBinding->kind() == QQmlAbstractBinding::ValueTypeProxy)
         oldBinding = static_cast<QQmlValueTypeProxyBinding *>(oldBinding.data())->binding(index);
 
     if (!oldBinding)
@@ -873,9 +912,8 @@ QQmlPropertyPrivate::binding(QObject *object, QQmlPropertyIndex index)
         binding = binding->nextBinding();
 
     if (binding && valueTypeIndex != -1) {
-        if (binding->isValueTypeProxy()) {
+        if (binding->kind() == QQmlAbstractBinding::ValueTypeProxy)
             binding = static_cast<QQmlValueTypeProxyBinding *>(binding)->binding(index);
-        }
     }
 
     return binding;
@@ -890,7 +928,7 @@ void QQmlPropertyPrivate::findAliasTarget(QObject *object, QQmlPropertyIndex bin
         int coreIndex = bindingIndex.coreIndex();
         int valueTypeIndex = bindingIndex.valueTypeIndex();
 
-        QQmlPropertyData *propertyData =
+        const QQmlPropertyData *propertyData =
             data->propertyCache?data->propertyCache->property(coreIndex):nullptr;
         if (propertyData && propertyData->isAlias()) {
             QQmlVMEMetaObject *vme = QQmlVMEMetaObject::getForProperty(object, coreIndex);
@@ -938,7 +976,7 @@ void QQmlPropertyPrivate::setBinding(QQmlAbstractBinding *binding, BindingFlags
     int coreIndex = index.coreIndex();
     QQmlData *data = QQmlData::get(object, true);
     if (data->propertyCache) {
-        QQmlPropertyData *propertyData = data->propertyCache->property(coreIndex);
+        const QQmlPropertyData *propertyData = data->propertyCache->property(coreIndex);
         Q_ASSERT(propertyData);
     }
 #endif
@@ -1098,7 +1136,7 @@ QVariant QQmlPropertyPrivate::readValueProperty()
 {
     auto doRead = [&](QQmlGadgetPtrWrapper *wrapper) {
         wrapper->read(object, core.coreIndex());
-        return wrapper->property(valueTypeData.coreIndex()).read(wrapper);
+        return wrapper->readOnGadget(wrapper->property(valueTypeData.coreIndex()));
     };
 
     if (isValueType()) {
@@ -1113,7 +1151,7 @@ QVariant QQmlPropertyPrivate::readValueProperty()
 
         QQmlListProperty<QObject> prop;
         core.readProperty(object, &prop);
-        return QVariant::fromValue(QQmlListReferencePrivate::init(prop, core.propType(), engine));
+        return QVariant::fromValue(QQmlListReferencePrivate::init(prop, core.propType()));
 
     } else if (core.isQObject()) {
 
@@ -1187,11 +1225,7 @@ bool QQmlPropertyPrivate::writeEnumProperty(const QMetaProperty &prop, int idx,
     QVariant v = value;
     if (prop.isEnumType()) {
         QMetaEnum menum = prop.enumerator();
-        if (v.userType() == QMetaType::QString
-#ifdef QT3_SUPPORT
-            || v.userType() == QVariant::CString
-#endif
-            ) {
+        if (v.userType() == QMetaType::QString) {
             bool ok;
             if (prop.isFlagType())
                 v = QVariant(menum.keysToValue(value.toByteArray(), &ok));
@@ -1267,8 +1301,7 @@ bool QQmlPropertyPrivate::write(
         const QQmlRefPointer<QQmlContextData> &context, QQmlPropertyData::WriteFlags flags)
 {
     const QMetaType propertyMetaType = property.propType();
-    const int propertyType = propertyMetaType.id();
-    const int variantType = value.userType();
+    const QMetaType variantMetaType = value.metaType();
 
     // we need to prevent new-style bindings from being  removed
     QUntypedPropertyBinding untypedBinding;
@@ -1292,7 +1325,7 @@ bool QQmlPropertyPrivate::write(
         QMetaProperty prop = object->metaObject()->property(property.coreIndex());
         QVariant v = value;
         // Enum values come through the script engine as doubles
-        if (variantType == QMetaType::Double) {
+        if (variantMetaType == QMetaType::fromType<double>()) {
             double integral;
             double fractional = std::modf(value.toDouble(), &integral);
             if (qFuzzyIsNull(fractional))
@@ -1302,28 +1335,28 @@ bool QQmlPropertyPrivate::write(
     }
 
     QQmlEnginePrivate *enginePriv = QQmlEnginePrivate::get(context);
-    const bool isUrl = propertyType == QMetaType::QUrl; // handled separately
+    const bool isUrl = propertyMetaType == QMetaType::fromType<QUrl>(); // handled separately
 
     // The cases below are in approximate order of likelyhood:
-    if (propertyType == variantType && !isUrl && propertyType != qMetaTypeId<QList<QUrl>>() && !property.isQList()) {
+    if (propertyMetaType == variantMetaType && !isUrl
+            && propertyMetaType != QMetaType::fromType<QList<QUrl>>() && !property.isQList()) {
         return property.writeProperty(object, const_cast<void *>(value.constData()), flags);
     } else if (property.isQObject()) {
         QVariant val = value;
-        const QMetaType variantMetaType = value.metaType();
         QMetaType varType;
         if (variantMetaType == QMetaType::fromType<std::nullptr_t>()) {
             // This reflects the fact that you can assign a nullptr to a QObject pointer
             // Without the change to QObjectStar, rawMetaObjectForType would not give us a QQmlMetaObject
             varType = QMetaType::fromType<QObject*>();
-            val = QVariant(QMetaType::fromType<QObject *>(), nullptr);
+            val = QVariant(varType, nullptr);
         } else {
             varType = variantMetaType;
         }
-        QQmlMetaObject valMo = rawMetaObjectForType(enginePriv, varType.id());
+        QQmlMetaObject valMo = rawMetaObjectForType(varType);
         if (valMo.isNull() || !varType.flags().testFlag(QMetaType::PointerToQObject))
             return false;
         QObject *o = *static_cast<QObject *const *>(val.constData());
-        QQmlMetaObject propMo = rawMetaObjectForType(enginePriv, propertyType);
+        QQmlMetaObject propMo = rawMetaObjectForType(propertyMetaType);
 
         if (o)
             valMo = o;
@@ -1339,10 +1372,10 @@ bool QQmlPropertyPrivate::write(
             return false;
         }
     } else if (value.canConvert(propertyMetaType)
-               && !isUrl && variantType != QMetaType::QString
-               && propertyType != qMetaTypeId<QList<QUrl>>() && !property.isQList()) {
+               && !isUrl && variantMetaType != QMetaType::fromType<QString>()
+               && propertyMetaType != QMetaType::fromType<QList<QUrl>>() && !property.isQList()) {
         // common cases:
-        switch (propertyType) {
+        switch (propertyMetaType.id()) {
         case QMetaType::Bool: {
             bool b = value.toBool();
             return property.writeProperty(object, &b, flags);
@@ -1369,81 +1402,100 @@ bool QQmlPropertyPrivate::write(
             return property.writeProperty(object, const_cast<void *>(v.constData()), flags);
         }
         }
-    } else if (propertyType == qMetaTypeId<QVariant>()) {
+    } else if (propertyMetaType == QMetaType::fromType<QVariant>()) {
         return property.writeProperty(object, const_cast<QVariant *>(&value), flags);
     } else if (isUrl) {
         QUrl u;
-        if (variantType == QMetaType::QUrl) {
+        if (variantMetaType == QMetaType::fromType<QUrl>()) {
             u = value.toUrl();
             if (compatResolveUrlsOnAssigment() && context && u.isRelative() && !u.isEmpty())
                 u = context->resolvedUrl(u);
         }
-        else if (variantType == QMetaType::QByteArray)
+        else if (variantMetaType == QMetaType::fromType<QByteArray>())
             u = QUrl(QString::fromUtf8(value.toByteArray()));
-        else if (variantType == QMetaType::QString)
+        else if (variantMetaType == QMetaType::fromType<QString>())
             u = QUrl(value.toString());
         else
             return false;
 
         return property.writeProperty(object, &u, flags);
-    } else if (propertyType == qMetaTypeId<QList<QUrl>>()) {
+    } else if (propertyMetaType == QMetaType::fromType<QList<QUrl>>()) {
         QList<QUrl> urlSeq = compatResolveUrlsOnAssigment()
                 ? urlSequence(value, context)
                 : urlSequence(value);
         return property.writeProperty(object, &urlSeq, flags);
     } else if (property.isQList()) {
-        QQmlMetaObject listType;
-
-        if (enginePriv) {
-            listType = enginePriv->rawMetaObjectForType(QQmlMetaType::listType(propertyMetaType).id());
-        } else {
-            QQmlType type = QQmlMetaType::qmlType(QQmlMetaType::listType(propertyMetaType).id());
-            if (!type.isValid())
+        if (propertyMetaType.flags() & QMetaType::IsQmlList) {
+            QMetaType listValueType = QQmlMetaType::listValueType(propertyMetaType);
+            QQmlMetaObject valueMetaObject = QQmlMetaType::rawMetaObjectForType(listValueType);
+            if (valueMetaObject.isNull())
                 return false;
-            listType = type.baseMetaObject();
-        }
-        if (listType.isNull())
-            return false;
 
-        QQmlListProperty<void> prop;
-        property.readProperty(object, &prop);
+            QQmlListProperty<void> prop;
+            property.readProperty(object, &prop);
 
-        if (!prop.clear)
-            return false;
+            if (!prop.clear)
+                return false;
 
-        prop.clear(&prop);
+            prop.clear(&prop);
 
-        if (variantType == qMetaTypeId<QQmlListReference>()) {
-            QQmlListReference qdlr = value.value<QQmlListReference>();
+            if (variantMetaType == QMetaType::fromType<QQmlListReference>()) {
+                QQmlListReference qdlr = value.value<QQmlListReference>();
 
-            for (qsizetype ii = 0; ii < qdlr.count(); ++ii) {
-                QObject *o = qdlr.at(ii);
-                if (o && !QQmlMetaObject::canConvert(o, listType))
-                    o = nullptr;
-                prop.append(&prop, o);
-            }
-        } else if (variantType == qMetaTypeId<QList<QObject *> >()) {
-            const QList<QObject *> &list = qvariant_cast<QList<QObject *> >(value);
+                for (qsizetype ii = 0; ii < qdlr.count(); ++ii) {
+                    QObject *o = qdlr.at(ii);
+                    if (o && !QQmlMetaObject::canConvert(o, valueMetaObject))
+                        o = nullptr;
+                    prop.append(&prop, o);
+                }
+            } else if (variantMetaType == QMetaType::fromType<QList<QObject *>>()) {
+                const QList<QObject *> &list = qvariant_cast<QList<QObject *> >(value);
 
-            for (qsizetype ii = 0; ii < list.count(); ++ii) {
-                QObject *o = list.at(ii);
-                if (o && !QQmlMetaObject::canConvert(o, listType))
+                for (qsizetype ii = 0; ii < list.size(); ++ii) {
+                    QObject *o = list.at(ii);
+                    if (o && !QQmlMetaObject::canConvert(o, valueMetaObject))
+                        o = nullptr;
+                    prop.append(&prop, o);
+                }
+            } else {
+                QObject *o = QQmlMetaType::toQObject(value);
+                if (o && !QQmlMetaObject::canConvert(o, valueMetaObject))
                     o = nullptr;
                 prop.append(&prop, o);
             }
+        } else if (variantMetaType == propertyMetaType) {
+            QVariant v = value;
+            property.writeProperty(object, v.data(), flags);
         } else {
-            QObject *o = QQmlMetaType::toQObject(value);
-            if (o && !QQmlMetaObject::canConvert(o, listType))
-                o = nullptr;
-            prop.append(&prop, o);
+            QVariant list(propertyMetaType);
+            const QQmlType type = QQmlMetaType::qmlType(propertyMetaType);
+            const QMetaSequence sequence = type.listMetaSequence();
+            if (sequence.canAddValue())
+                sequence.addValue(list.data(), value.data());
+            property.writeProperty(object, list.data(), flags);
         }
+    } else if (variantMetaType == QMetaType::fromType<QJSValue>()) {
+        QJSValue jsValue = qvariant_cast<QJSValue>(value);
+        const QV4::FunctionObject *f
+                = QJSValuePrivate::asManagedType<QV4::FunctionObject>(&jsValue);
+        if (f && f->isBinding()) {
+            QV4::QObjectWrapper::setProperty(
+                    f->engine(), object, &property, f->asReturnedValue());
+            return true;
+        }
+        return false;
+    } else if (enginePriv && propertyMetaType == QMetaType::fromType<QJSValue>()) {
+        // We can convert everything into a QJSValue if we have an engine.
+        QJSValue jsValue = QJSValuePrivate::fromReturnedValue(
+                    enginePriv->v4engine()->metaTypeToJS(variantMetaType, value.constData()));
+        return property.writeProperty(object, &jsValue, flags);
     } else {
-        Q_ASSERT(variantType != propertyType);
+        Q_ASSERT(variantMetaType != propertyMetaType);
 
         bool ok = false;
         QVariant v;
-        if (variantType == QMetaType::QString)
-            v = QQmlStringConverters::variantFromString(value.toString(), propertyType, &ok);
+        if (variantMetaType == QMetaType::fromType<QString>())
+            v = QQmlStringConverters::variantFromString(value.toString(), propertyMetaType, &ok);
 
         if (!ok) {
             v = value;
@@ -1456,28 +1508,33 @@ bool QQmlPropertyPrivate::write(
             // to a sequence type property (eg, an int to a QList<int> property).
             // or that we encountered an interface type
             // Note that we've already handled single-value assignment to QList<QUrl> properties.
-            if (variantType == QMetaType::Int && propertyType == qMetaTypeId<QList<int> >()) {
+            if (variantMetaType == QMetaType::fromType<int>()
+                    && propertyMetaType == QMetaType::fromType<QList<int>>()) {
                 QList<int> list;
                 list << value.toInt();
                 v = QVariant::fromValue<QList<int> >(list);
                 ok = true;
-            } else if ((variantType == QMetaType::Double || variantType == QMetaType::Int)
-                       && (propertyType == qMetaTypeId<QList<qreal> >())) {
+            } else if ((variantMetaType == QMetaType::fromType<double>()
+                        || variantMetaType == QMetaType::fromType<int>())
+                       && (propertyMetaType == QMetaType::fromType<QList<qreal>>())) {
                 QList<qreal> list;
                 list << value.toReal();
                 v = QVariant::fromValue<QList<qreal> >(list);
                 ok = true;
-            } else if (variantType == QMetaType::Bool && propertyType == qMetaTypeId<QList<bool> >()) {
+            } else if (variantMetaType == QMetaType::fromType<bool>()
+                       && propertyMetaType == QMetaType::fromType<QList<bool>>()) {
                 QList<bool> list;
                 list << value.toBool();
                 v = QVariant::fromValue<QList<bool> >(list);
                 ok = true;
-            } else if (variantType == QMetaType::QString && propertyType == qMetaTypeId<QList<QString> >()) {
+            } else if (variantMetaType == QMetaType::fromType<QString>()
+                       && propertyMetaType == QMetaType::fromType<QList<QString>>()) {
                 QList<QString> list;
                 list << value.toString();
                 v = QVariant::fromValue<QList<QString> >(list);
                 ok = true;
-            } else if (variantType == QMetaType::QString && propertyType == qMetaTypeId<QStringList>()) {
+            } else if (variantMetaType == QMetaType::fromType<QString>()
+                       && propertyMetaType == QMetaType::fromType<QStringList>()) {
                 QStringList list;
                 list << value.toString();
                 v = QVariant::fromValue<QStringList>(list);
@@ -1485,11 +1542,11 @@ bool QQmlPropertyPrivate::write(
             }
         }
 
-        if (!ok && QQmlMetaType::isInterface(propertyType)) {
+        if (!ok && QQmlMetaType::isInterface(propertyMetaType)) {
             auto valueAsQObject = qvariant_cast<QObject *>(value);
 
             if (void *interface = valueAsQObject
-                        ? valueAsQObject->qt_metacast(QQmlMetaType::interfaceIId(propertyType))
+                        ? valueAsQObject->qt_metacast(QQmlMetaType::interfaceIId(propertyMetaType))
                         : nullptr;
                 interface) {
                 // this case can occur when object has an interface type
@@ -1508,17 +1565,13 @@ bool QQmlPropertyPrivate::write(
     return true;
 }
 
-QQmlMetaObject QQmlPropertyPrivate::rawMetaObjectForType(QQmlEnginePrivate *engine, int userType)
+QQmlMetaObject QQmlPropertyPrivate::rawMetaObjectForType(QMetaType metaType)
 {
-    QMetaType metaType(userType);
-    if ((metaType.flags() & QMetaType::PointerToQObject) && metaType.metaObject())
-        return metaType.metaObject();
-    if (engine)
-        return engine->rawMetaObjectForType(userType);
-    QQmlType type = QQmlMetaType::qmlType(userType);
-    if (type.isValid())
-        return QQmlMetaObject(type.baseMetaObject());
-    return QQmlMetaObject();
+    if (metaType.flags() & QMetaType::PointerToQObject) {
+        if (const QMetaObject *metaObject = metaType.metaObject())
+            return metaObject;
+    }
+    return QQmlMetaType::rawMetaObjectForType(metaType);
 }
 
 /*!
@@ -1735,7 +1788,7 @@ QMetaMethod QQmlPropertyPrivate::findSignalByName(const QMetaObject *mo, const Q
     // If no signal is found, but the signal is of the form "onBlahChanged",
     // return the notify signal for the property "Blah"
     if (name.endsWith("Changed")) {
-        QByteArray propName = name.mid(0, name.length() - 7);
+        QByteArray propName = name.mid(0, name.size() - 7);
         int propIdx = mo->indexOfProperty(propName.constData());
         if (propIdx >= 0) {
             QMetaProperty prop = mo->property(propIdx);
@@ -1747,6 +1800,16 @@ QMetaMethod QQmlPropertyPrivate::findSignalByName(const QMetaObject *mo, const Q
     return QMetaMethod();
 }
 
+/*!
+    Return the property corresponding to \a name
+*/
+QMetaProperty QQmlPropertyPrivate::findPropertyByName(const QMetaObject *mo, const QByteArray &name)
+{
+    Q_ASSERT(mo);
+    const int i = mo->indexOfProperty(name);
+    return i < 0 ? QMetaProperty() : mo->property(i);
+}
+
 /*! \internal
     If \a indexInSignalRange is true, \a index is treated as a signal index
     (see QObjectPrivate::signalIndex()), otherwise it is treated as a
@@ -1756,7 +1819,7 @@ static inline void flush_vme_signal(const QObject *object, int index, bool index
 {
     QQmlData *data = QQmlData::get(object);
     if (data && data->propertyCache) {
-        QQmlPropertyData *property = indexInSignalRange ? data->propertyCache->signal(index)
+        const QQmlPropertyData *property = indexInSignalRange ? data->propertyCache->signal(index)
                                                         : data->propertyCache->method(index);
 
         if (property && property->isVMESignal()) {
@@ -1799,3 +1862,5 @@ void QQmlPropertyPrivate::flushSignal(const QObject *sender, int signal_index)
 }
 
 QT_END_NAMESPACE
+
+#include "moc_qqmlproperty.cpp"
