@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtQuick module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include <private/qsgadaptationlayer_p.h>
 #include "qquickcanvasitem_p.h"
@@ -288,7 +252,7 @@ QQuickCanvasItemPrivate::~QQuickCanvasItemPrivate()
     QPainter instead of the more expensive and likely less performing
     JavaScript and Context2D approach.
 
-    \sa Context2D QQuickPaintedItem
+    \sa Context2D, QQuickPaintedItem
 */
 
 QQuickCanvasItem::QQuickCanvasItem(QQuickItem *parent)
@@ -972,22 +936,25 @@ void QQuickCanvasItem::checkAnimationCallbacks()
 }
 
 /*!
-    \qmlmethod bool QtQuick::Canvas::save(string filename)
+    \qmlmethod bool QtQuick::Canvas::save(string filename, size imageSize = undefined)
 
     Saves the current canvas content into an image file \a filename.
-    The saved image format is automatically decided by the \a filename's
-    suffix. Returns \c true on success.
+    The saved image format is automatically decided by the \a filename's suffix.
+    Returns \c true on success. If \a imageSize is specified, the resulting
+    image will have this size, and will have a devicePixelRatio of \c 1.0.
+    Otherwise, the \l {QQuickWindow::}{devicePixelRatio()} of the window in
+    which the canvas is displayed is applied to the saved image.
 
     \note Calling this method will force painting the whole canvas, not just the
     current canvas visible window.
 
     \sa canvasWindow, canvasSize, toDataURL()
 */
-bool QQuickCanvasItem::save(const QString &filename) const
+bool QQuickCanvasItem::save(const QString &filename, const QSizeF &imageSize) const
 {
     Q_D(const QQuickCanvasItem);
     QUrl url = d->baseUrl.resolved(QUrl::fromLocalFile(filename));
-    return toImage().save(url.toLocalFile());
+    return toImage(QRectF(QPointF(0, 0), imageSize)).save(url.toLocalFile());
 }
 
 QQmlRefPointer<QQuickCanvasPixmap> QQuickCanvasItem::loadedPixmap(const QUrl& url)
@@ -1097,6 +1064,12 @@ bool QQuickCanvasItem::isImageLoaded(const QUrl& url) const
         && d->pixmaps.value(fullPathUrl)->pixmap()->isReady();
 }
 
+/*!
+    \internal
+    Returns a QImage representing the requested \a rect which is in device independent pixels of the item.
+    If \a rect is empty, then it will use the whole item's rect by default.
+*/
+
 QImage QQuickCanvasItem::toImage(const QRectF& rect) const
 {
     Q_D(const QQuickCanvasItem);
@@ -1105,7 +1078,7 @@ QImage QQuickCanvasItem::toImage(const QRectF& rect) const
         return QImage();
 
     const QRectF &rectSource = rect.isEmpty() ? canvasWindow() : rect;
-    const qreal dpr = window() ? window()->effectiveDevicePixelRatio() : qreal(1);
+    const qreal dpr = window() && rect.isEmpty() ? window()->effectiveDevicePixelRatio() : qreal(1);
     const QRectF rectScaled(rectSource.topLeft() * dpr, rectSource.size() * dpr);
 
     QImage image = d->context->toImage(rectScaled);
