@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtQuick module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qquickrendercontrol.h"
 #include "qquickrendercontrol_p.h"
@@ -157,7 +121,8 @@ QQuickRenderControlPrivate::QQuickRenderControlPrivate(QQuickRenderControl *rend
       ownRhi(true),
       cb(nullptr),
       offscreenSurface(nullptr),
-      sampleCount(1)
+      sampleCount(1),
+      frameStatus(NotRecordingFrame)
 {
     if (!sg) {
         qAddPostRoutine(cleanup);
@@ -210,7 +175,12 @@ QQuickRenderControl::~QQuickRenderControl()
 
     delete d->rc;
 
-    d->resetRhi();
+    // Only call rhi related cleanup when we actually got to initialize() and
+    // managed to get a QRhi. The software backend for instance would mean
+    // using the rendercontrol without ever calling initialize() - it is then
+    // important to completely skip calling any QSGRhiSupport functions.
+    if (d->rhi)
+        d->resetRhi();
 }
 
 void QQuickRenderControlPrivate::windowDestroyed()
@@ -223,7 +193,7 @@ void QQuickRenderControlPrivate::windowDestroyed()
 
         QQuickWindowPrivate::get(window)->animationController.reset();
 #if QT_CONFIG(quick_shadereffect)
-        QSGRhiShaderEffectNode::cleanupMaterialTypeCache();
+        QSGRhiShaderEffectNode::cleanupMaterialTypeCache(window);
 #endif
         window = nullptr;
     }
@@ -428,6 +398,7 @@ void QQuickRenderControl::invalidate()
     // also essential to allow a subsequent initialize() to succeed.
     d->rc->invalidate();
 
+    d->frameStatus = QQuickRenderControlPrivate::NotRecordingFrame;
     d->initialized = false;
 }
 
@@ -635,7 +606,23 @@ void QQuickRenderControl::beginFrame()
 
     emit d->window->beforeFrameBegin();
 
-    d->rhi->beginOffscreenFrame(&d->cb);
+    QRhi::FrameOpResult result = d->rhi->beginOffscreenFrame(&d->cb);
+
+    switch (result) {
+    case QRhi::FrameOpSuccess:
+    case QRhi::FrameOpSwapChainOutOfDate:
+        d->frameStatus = QQuickRenderControlPrivate::RecordingFrame;
+        break;
+    case QRhi::FrameOpError:
+        d->frameStatus = QQuickRenderControlPrivate::ErrorInBeginFrame;
+        break;
+    case QRhi::FrameOpDeviceLost:
+        d->frameStatus = QQuickRenderControlPrivate::DeviceLostInBeginFrame;
+        break;
+    default:
+        d->frameStatus = QQuickRenderControlPrivate::NotRecordingFrame;
+        break;
+    }
 }
 
 /*!
@@ -658,6 +645,7 @@ void QQuickRenderControl::endFrame()
 
     d->rhi->endOffscreenFrame();
     d->cb = nullptr;
+    d->frameStatus = QQuickRenderControlPrivate::NotRecordingFrame;
 
     emit d->window->afterFrameEnd();
 }
@@ -665,8 +653,10 @@ void QQuickRenderControl::endFrame()
 bool QQuickRenderControlPrivate::initRhi()
 {
     // initialize() - invalidate() - initialize() uses the QRhi the first
-    // initialize() created, so if already exists, we are done
-    if (rhi)
+    // initialize() created, so if already exists, we are done. Does not apply
+    // when wrapping an externally created QRhi, because we may be associated
+    // with a new one now.
+    if (rhi && ownRhi)
         return true;
 
     QSGRhiSupport *rhiSupport = QSGRhiSupport::instance();
