@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2021 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtQuick module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2021 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qquicktreeview_p_p.h"
 
@@ -72,7 +36,7 @@
 
     Even if TreeViewDelegate is customizable, there might be situations
     where you want to render the tree in a different way, or ensure that
-    the delegate ends up as mininal as possible, e.g for performance reasons.
+    the delegate ends up as minimal as possible, perhaps for performance reasons.
     Creating your own delegate from scratch is easy, since TreeView offers
     a set of properties that can be used to position and render each node
     in the tree correctly.
@@ -126,7 +90,7 @@
     If \a row is not between \c 0 and \l {TableView::}{rows}, the return value will
     be \c -1.
 
-    \sa modelIndex()
+    \sa {TableView::}{modelIndex()}
 */
 
 /*!
@@ -149,7 +113,48 @@
     \note this function will not affect the model, only
     the visual representation in the view.
 
-    \sa collapse(), isExpanded()
+    \sa collapse(), isExpanded(), expandRecursively()
+*/
+
+/*!
+    \qmlmethod QtQuick::TreeView::expandRecursively(row = -1, depth = -1)
+    \since 6.4
+
+    Expands the tree node at the given \a row in the view recursively down to
+    \a depth. \a depth should be relative to the depth of \a row. If
+    \a depth is \c -1, the tree will be expanded all the way down to all leaves.
+
+    For a model that has more than one root, you can also call this function
+    with \a row equal to \c -1. This will expand all roots. Hence, calling
+    expandRecursively(-1, -1), or simply expandRecursively(), will expand
+    all nodes in the model.
+
+    \a row should be the row in the view (table row), and not a row in the model.
+
+    \note This function will not try to \l{QAbstractItemModel::fetchMore}{fetch more} data.
+    \note This function will not affect the model, only the visual representation in the view.
+    \warning If the model contains a large number of items, this function will
+    take some time to execute.
+
+    \sa collapseRecursively(), expand(), collapse(), isExpanded(), depth()
+*/
+
+/*!
+    \qmlmethod QtQuick::TreeView::expandToIndex(QModelIndex index)
+    \since 6.4
+
+    Expands the tree from the given model \a index, and recursively all the way up
+    to the root. The result will be that the delegate item that represents \a index
+    becomes visible in the view (unless it ends up outside the viewport). To
+    ensure that the row ends up visible in the viewport, you can do:
+
+    \code
+        expandToIndex(index)
+        forceLayout()
+        positionViewAtRow(rowAtIndex(index), Qt.AlignVCenter)
+    \endcode
+
+    \sa expand(), expandRecursively()
 */
 
 /*!
@@ -165,6 +170,26 @@
     \sa expand(), isExpanded()
 */
 
+/*!
+    \qmlmethod QtQuick::TreeView::collapseRecursively(row = -1)
+    \since 6.4
+
+    Collapses the tree node at the given \a row in the view recursively down to
+    all leaves.
+
+    For a model has more than one root, you can also call this function
+    with \a row equal to \c -1. This will collapse all roots. Hence, calling
+    collapseRecursively(-1), or simply collapseRecursively(), will collapse
+    all nodes in the model.
+
+    \a row should be the row in the view (table row), and not a row in the model.
+
+    \note this function will not affect the model, only
+    the visual representation in the view.
+
+    \sa expandRecursively(), expand(), collapse(), isExpanded(), depth()
+*/
+
 /*!
     \qmlmethod QtQuick::TreeView::toggleExpanded(row)
 
@@ -182,93 +207,35 @@
 */
 
 /*!
-    \qmlmethod QModelIndex QtQuick::TreeView::modelIndex(row, column)
-
-    Returns the \l QModelIndex that maps to \a row and \a column in the view.
-
-    \a row and \a column should be the row and column in the view (table row and
-    table column), and not a row and column in the model.
-
-    The assigned model, which is a tree model, is converted to a flat table
-    model internally so that it can be shown in a TableView (which TreeView
-    inherits). This function can be used whenever you need to know which
-    index in the tree model maps to the given row and column in the view.
-
-    \sa rowAtIndex(), columnAtIndex()
-*/
-
-/*!
-    \qmlmethod QModelIndex QtQuick::TreeView::modelIndex(point cell)
-
-    Convenience function for doing:
-    \code
-    modelIndex(cell.y, cell.x)
-    \endcode
-
-    A cell is simply a \l point that combines row and column into
-    a single type. Note that \c point.x will map to the column, and
-    \c point.y will map to the row.
-*/
-
-/*!
-    \qmlmethod int QtQuick::TreeView::rowAtIndex(modelIndex)
-
-    Returns the row in the view that maps to \a modelIndex in the model.
-
-    The assigned model, which is a tree model, is converted to a flat table
-    model internally so that it can be shown in a TableView (which TreeView
-    inherits). This function can be used whenever you need to know which
-    row in the view the given model index maps to.
-
-    \note \a modelIndex must be a \l QModelIndex.
-
-    \sa columnAtIndex(), modelIndex()
-*/
-
-/*!
-    \qmlmethod int QtQuick::TreeView::columnAtIndex(modelIndex)
-
-    Returns the column in the view that maps to \a modelIndex in the model.
-
-    The assigned model, which is a tree model, is converted to a flat table
-    model internally so that it can be shown in a TableView (which TreeView
-    inherits). This function can be used whenever you need to know which
-    column in the view the given model index maps to.
-
-    \note \a modelIndex must be a \l QModelIndex.
-
-    \sa rowAtIndex(), modelIndex()
-*/
-
-/*!
-    \qmlmethod point QtQuick::TreeView::cellAtIndex(modelIndex)
-
-    Convenience function for doing:
-
-    \c {Qt.point(columnAtIndex(}\a {modelIndex}\c{), rowAtIndex(}\a {modelIndex}\c{))}
-
-    A cell is simply a \l point that combines row and column into
-    a single type. Note that \c point.x will map to the column, and
-    \c point.y will map to the row.
-*/
-
-/*!
-    \qmlsignal QtQuick::TreeView::expanded(row)
+    \qmlsignal QtQuick::TreeView::expanded(row, depth)
 
     This signal is emitted when a \a row is expanded in the view.
+    \a row and \a depth will be equal to the arguments given to the call
+    that caused the expansion to happen (\l expand() or \l expandRecursively()).
+    In case of \l expand(), \a depth will always be \c 1.
+    In case of \l expandToIndex(), \a depth will be the depth of the
+    target index.
+
+    \note when a row is expanded recursively, the expanded signal will
+    only be emitted for that one row, and not for its descendants.
 
     \sa collapsed(), expand(), collapse(), toggleExpanded()
 */
 
 /*!
-    \qmlsignal QtQuick::TreeView::collapsed(row)
+    \qmlsignal QtQuick::TreeView::collapsed(row, recursively)
 
     This signal is emitted when a \a row is collapsed in the view.
+    \a row will be equal to the argument given to the call that caused
+    the collapse to happen (\l collapse() or \l collapseRecursively()).
+    If the row was collapsed recursively, \a recursively will be \c true.
+
+    \note when a row is collapsed recursively, the collapsed signal will
+    only be emitted for that one row, and not for its descendants.
 
     \sa expanded(), expand(), collapse(), toggleExpanded()
 */
 
-static const char* kRequiredProperties = "_qt_treeview_requiredpropertymask";
 // Hard-code the tree column to be 0 for now
 static const int kTreeColumn = 0;
 
@@ -343,32 +310,6 @@ void QQuickTreeViewPrivate::dataChangedCallback(
     }
 }
 
-void QQuickTreeViewPrivate::setRequiredProperty(const char *property,
-    const QVariant &value, int serializedModelIndex, QObject *object, bool init)
-{
-    // Attaching a property list to the delegate item is just a
-    // work-around until QMetaProperty::isRequired() works!
-    const QString propertyName = QString::fromUtf8(property);
-
-    if (init) {
-        const bool wasRequired = model->setRequiredProperty(serializedModelIndex, propertyName, value);
-        if (wasRequired) {
-            QStringList propertyList = object->property(kRequiredProperties).toStringList();
-            object->setProperty(kRequiredProperties, propertyList << propertyName);
-        }
-    } else {
-        const QStringList propertyList = object->property(kRequiredProperties).toStringList();
-        if (!propertyList.contains(propertyName)) {
-            // We only write to properties that are required
-            return;
-        }
-        const auto metaObject = object->metaObject();
-        const int propertyIndex = metaObject->indexOfProperty(property);
-        const auto metaProperty = metaObject->property(propertyIndex);
-        metaProperty.write(object, value);
-    }
-}
-
 void QQuickTreeViewPrivate::updateRequiredProperties(int serializedModelIndex, QObject *object, bool init)
 {
     Q_Q(QQuickTreeView);
@@ -383,11 +324,69 @@ void QQuickTreeViewPrivate::updateRequiredProperties(int serializedModelIndex, Q
     setRequiredProperty("depth", m_treeModelToTableModel.depthAtRow(row), serializedModelIndex, object, init);
 }
 
+void QQuickTreeViewPrivate::updateSelection(const QRect &oldSelection, const QRect &newSelection)
+{
+    Q_Q(QQuickTreeView);
+
+    const QRect oldRect = oldSelection.normalized();
+    const QRect newRect = newSelection.normalized();
+
+    if (oldSelection == newSelection)
+        return;
+
+    // Select the rows inside newRect that doesn't overlap with oldRect
+    for (int row = newRect.y(); row <= newRect.y() + newRect.height(); ++row) {
+        if (oldRect.y() != -1 && oldRect.y() <= row && row <= oldRect.y() + oldRect.height())
+            continue;
+        const QModelIndex startIndex = q->modelIndex(newRect.x(), row);
+        const QModelIndex endIndex = q->modelIndex(newRect.x() + newRect.width(), row);
+        selectionModel->select(QItemSelection(startIndex, endIndex), QItemSelectionModel::Select);
+    }
+
+    if (oldRect.x() != -1) {
+        // Since oldRect is valid, this update is a continuation of an already existing selection!
+
+        // Select the columns inside newRect that don't overlap with oldRect
+        for (int column = newRect.x(); column <= newRect.x() + newRect.width(); ++column) {
+            if (oldRect.x() <= column && column <= oldRect.x() + oldRect.width())
+                continue;
+            for (int row = newRect.y(); row <= newRect.y() + newRect.height(); ++row)
+                selectionModel->select(q->modelIndex(column, row), QItemSelectionModel::Select);
+        }
+
+        // Unselect the rows inside oldRect that don't overlap with newRect
+        for (int row = oldRect.y(); row <= oldRect.y() + oldRect.height(); ++row) {
+            if (newRect.y() <= row && row <= newRect.y() + newRect.height())
+                continue;
+            const QModelIndex startIndex = q->modelIndex(oldRect.x(), row);
+            const QModelIndex endIndex = q->modelIndex(oldRect.x() + oldRect.width(), row);
+            selectionModel->select(QItemSelection(startIndex, endIndex), QItemSelectionModel::Deselect);
+        }
+
+        // Unselect the columns inside oldRect that don't overlap with newRect
+        for (int column = oldRect.x(); column <= oldRect.x() + oldRect.width(); ++column) {
+            if (newRect.x() <= column && column <= newRect.x() + newRect.width())
+                continue;
+            // Since we're not allowed to call select/unselect on the selectionModel with
+            // indices from different parents, and since indicies from different parents are
+            // expected when working with trees, we need to unselect the indices in the column
+            // one by one, rather than the whole column in one go. This, however, can cause a
+            // lot of selection fragments in the selectionModel, which eventually can hurt
+            // performance. But large selections containing a lot of columns is not normally
+            // the case for a treeview, so accept this potential corner case for now.
+            for (int row = newRect.y(); row <= newRect.y() + newRect.height(); ++row)
+                selectionModel->select(q->modelIndex(column, row), QItemSelectionModel::Deselect);
+        }
+    }
+}
+
 QQuickTreeView::QQuickTreeView(QQuickItem *parent)
     : QQuickTableView(*(new QQuickTreeViewPrivate), parent)
 {
     Q_D(QQuickTreeView);
 
+    setSelectionBehavior(SelectRows);
+
     // Note: QQuickTableView will only ever see the table model m_treeModelToTableModel, and
     // never the actual tree model that is assigned to us by the application.
     const auto modelAsVariant = QVariant::fromValue(std::addressof(d->m_treeModelToTableModel));
@@ -419,23 +418,98 @@ bool QQuickTreeView::isExpanded(int row) const
 }
 
 void QQuickTreeView::expand(int row)
+{
+    if (row >= 0)
+        expandRecursively(row, 1);
+}
+
+void QQuickTreeView::expandRecursively(int row, int depth)
 {
     Q_D(QQuickTreeView);
-    if (row < 0 || row >= d->m_treeModelToTableModel.rowCount())
+    if (row >= d->m_treeModelToTableModel.rowCount())
+        return;
+    if (row < 0 && row != -1)
+        return;
+    if (depth == 0 || depth < -1)
         return;
 
-    if (d->m_treeModelToTableModel.isExpanded(row))
-        return;
+    auto expandRowRecursively = [this, d, depth](int startRow) {
+        d->m_treeModelToTableModel.expandRecursively(startRow, depth);
+        // Update the expanded state of the startRow. The descendant rows that gets
+        // expanded will get the correct state set from initItem/itemReused instead.
+        for (int c = leftColumn(); c <= rightColumn(); ++c) {
+            const QPoint treeNodeCell(c, startRow);
+            if (const auto item = itemAtCell(treeNodeCell))
+                d->setRequiredProperty("expanded", true, d->modelIndexAtCell(treeNodeCell), item, false);
+        }
+    };
 
-    d->m_treeModelToTableModel.expandRow(row);
-
-    for (int c = leftColumn(); c <= rightColumn(); ++c) {
-        const QPoint treeNodeCell(c, row);
-        if (const auto item = itemAtCell(treeNodeCell))
-            d->setRequiredProperty("expanded", true, d->modelIndexAtCell(treeNodeCell), item, false);
+    if (row >= 0) {
+        // Expand only one row recursively
+        const bool isExpanded = d->m_treeModelToTableModel.isExpanded(row);
+        if (isExpanded && depth == 1)
+            return;
+        expandRowRecursively(row);
+    } else {
+        // Expand all root nodes recursively
+        const auto model = d->m_treeModelToTableModel.model();
+        for (int r = 0; r < model->rowCount(); ++r) {
+            const int rootRow = d->m_treeModelToTableModel.itemIndex(model->index(r, 0));
+            if (rootRow != -1)
+                expandRowRecursively(rootRow);
+        }
     }
 
-    emit expanded(row);
+    emit expanded(row, depth);
+}
+
+void QQuickTreeView::expandToIndex(const QModelIndex &index)
+{
+    Q_D(QQuickTreeView);
+
+    if (!index.isValid()) {
+        qmlWarning(this) << "index is not valid: " << index;
+        return;
+    }
+
+    if (index.model() != d->m_treeModelToTableModel.model()) {
+        qmlWarning(this) << "index doesn't belong to correct model: " << index;
+        return;
+    }
+
+    if (rowAtIndex(index) != -1) {
+        // index is already visible
+        return;
+    }
+
+    int depth = 1;
+    QModelIndex parent = index.parent();
+    int row = rowAtIndex(parent);
+
+    while (parent.isValid()) {
+        if (row != -1) {
+            // The node is already visible, since it maps to a row in the table!
+            d->m_treeModelToTableModel.expandRow(row);
+
+            // Update the state of the already existing delegate item
+            for (int c = leftColumn(); c <= rightColumn(); ++c) {
+                const QPoint treeNodeCell(c, row);
+                if (const auto item = itemAtCell(treeNodeCell))
+                    d->setRequiredProperty("expanded", true, d->modelIndexAtCell(treeNodeCell), item, false);
+            }
+
+            // When we hit a node that is visible, we know that all other nodes
+            // up to the parent have to be visible as well, so we can stop.
+            break;
+        } else {
+            d->m_treeModelToTableModel.expand(parent);
+            parent = parent.parent();
+            row = rowAtIndex(parent);
+            depth++;
+        }
+    }
+
+    emit expanded(row, depth);
 }
 
 void QQuickTreeView::collapse(int row)
@@ -455,7 +529,42 @@ void QQuickTreeView::collapse(int row)
             d->setRequiredProperty("expanded", false, d->modelIndexAtCell(treeNodeCell), item, false);
     }
 
-    emit collapsed(row);
+    emit collapsed(row, false);
+}
+
+void QQuickTreeView::collapseRecursively(int row)
+{
+    Q_D(QQuickTreeView);
+    if (row >= d->m_treeModelToTableModel.rowCount())
+        return;
+    if (row < 0 && row != -1)
+        return;
+
+    auto collapseRowRecursive = [this, d](int startRow) {
+        // Always collapse descendants recursively,
+        // even if the top row itself is already collapsed.
+        d->m_treeModelToTableModel.collapseRecursively(startRow);
+        // Update the expanded state of the (still visible) startRow
+        for (int c = leftColumn(); c <= rightColumn(); ++c) {
+            const QPoint treeNodeCell(c, startRow);
+            if (const auto item = itemAtCell(treeNodeCell))
+                d->setRequiredProperty("expanded", false, d->modelIndexAtCell(treeNodeCell), item, false);
+        }
+    };
+
+    if (row >= 0) {
+        collapseRowRecursive(row);
+    } else {
+        // Collapse all root nodes recursively
+        const auto model = d->m_treeModelToTableModel.model();
+        for (int r = 0; r < model->rowCount(); ++r) {
+            const int rootRow = d->m_treeModelToTableModel.itemIndex(model->index(r, 0));
+            if (rootRow != -1)
+                collapseRowRecursive(rootRow);
+        }
+    }
+
+    emit collapsed(row, true);
 }
 
 void QQuickTreeView::toggleExpanded(int row)
@@ -466,26 +575,11 @@ void QQuickTreeView::toggleExpanded(int row)
         expand(row);
 }
 
-QModelIndex QQuickTreeView::modelIndex(int row, int column) const
-{
-    Q_D(const QQuickTreeView);
-    const QModelIndex tableIndex = d->m_treeModelToTableModel.index(row, column);
-    return d->m_treeModelToTableModel.mapToModel(tableIndex);
-}
-
 QModelIndex QQuickTreeView::modelIndex(const QPoint &cell) const
 {
-    return modelIndex(cell.y(), cell.x());
-}
-
-int QQuickTreeView::rowAtIndex(const QModelIndex &index) const
-{
-    return d_func()->m_treeModelToTableModel.mapFromModel(index).row();
-}
-
-int QQuickTreeView::columnAtIndex(const QModelIndex &index) const
-{
-    return d_func()->m_treeModelToTableModel.mapFromModel(index).column();
+    Q_D(const QQuickTreeView);
+    const QModelIndex tableIndex = d->m_treeModelToTableModel.index(cell.y(), cell.x());
+    return d->m_treeModelToTableModel.mapToModel(tableIndex);
 }
 
 QPoint QQuickTreeView::cellAtIndex(const QModelIndex &index) const
@@ -494,6 +588,38 @@ QPoint QQuickTreeView::cellAtIndex(const QModelIndex &index) const
     return QPoint(tableIndex.column(), tableIndex.row());
 }
 
+QModelIndex QQuickTreeView::modelIndex(int column, int row) const
+{
+    return modelIndex({column, row});
+}
+
+void QQuickTreeView::keyPressEvent(QKeyEvent *event)
+{
+    event->ignore();
+
+    if (!keyNavigationEnabled())
+        return;
+    if (!selectionModel())
+        return;
+
+    const int row = cellAtIndex(selectionModel()->currentIndex()).y();
+    switch (event->key()) {
+    case Qt::Key_Left:
+        collapse(row);
+        event->accept();
+        break;
+    case Qt::Key_Right:
+        expand(row);
+        event->accept();
+        break;
+    default:
+        break;
+    }
+
+    if (!event->isAccepted())
+        QQuickTableView::keyPressEvent(event);
+}
+
 QT_END_NAMESPACE
 
 #include "moc_qquicktreeview_p.cpp"
