@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtQuick module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qquickstategroup_p.h"
 
@@ -53,7 +17,9 @@
 
 QT_BEGIN_NAMESPACE
 
-DEFINE_BOOL_CONFIG_OPTION(stateChangeDebug, STATECHANGE_DEBUG);
+using namespace Qt::StringLiterals;
+
+Q_DECLARE_LOGGING_CATEGORY(lcStates)
 
 class QQuickStateGroupPrivate : public QObjectPrivate
 {
@@ -130,7 +96,7 @@ QQuickStateGroup::QQuickStateGroup(QObject *parent)
 QQuickStateGroup::~QQuickStateGroup()
 {
     Q_D(const QQuickStateGroup);
-    for (int i = 0; i < d->states.count(); ++i)
+    for (int i = 0; i < d->states.size(); ++i)
         d->states.at(i)->setStateGroup(nullptr);
     if (d->nullState)
         d->nullState->setStateGroup(nullptr);
@@ -187,7 +153,7 @@ void QQuickStateGroupPrivate::append_state(QQmlListProperty<QQuickState> *list,
 qsizetype QQuickStateGroupPrivate::count_state(QQmlListProperty<QQuickState> *list)
 {
     QQuickStateGroup *_this = static_cast<QQuickStateGroup *>(list->object);
-    return _this->d_func()->states.count();
+    return _this->d_func()->states.size();
 }
 
 QQuickState *QQuickStateGroupPrivate::at_state(QQmlListProperty<QQuickState> *list, qsizetype index)
@@ -200,7 +166,7 @@ void QQuickStateGroupPrivate::clear_states(QQmlListProperty<QQuickState> *list)
 {
     QQuickStateGroup *_this = static_cast<QQuickStateGroup *>(list->object);
     _this->d_func()->setCurrentStateInternal(QString(), true);
-    for (qsizetype i = 0; i < _this->d_func()->states.count(); ++i) {
+    for (qsizetype i = 0; i < _this->d_func()->states.size(); ++i) {
         _this->d_func()->states.at(i)->setStateGroup(nullptr);
     }
     _this->d_func()->states.clear();
@@ -224,7 +190,7 @@ void QQuickStateGroupPrivate::removeLast_states(QQmlListProperty<QQuickState> *l
 {
     auto *d = qobject_cast<QQuickStateGroup *>(list->object)->d_func();
     if (d->currentState == d->states.last()->name())
-        d->setCurrentStateInternal(d->states.length() > 1 ? d->states.first()->name() : QString(), true);
+        d->setCurrentStateInternal(d->states.size() > 1 ? d->states.first()->name() : QString(), true);
     d->states.last()->setStateGroup(nullptr);
     d->states.removeLast();
 }
@@ -268,7 +234,7 @@ void QQuickStateGroupPrivate::append_transition(QQmlListProperty<QQuickTransitio
 qsizetype QQuickStateGroupPrivate::count_transitions(QQmlListProperty<QQuickTransition> *list)
 {
     QQuickStateGroup *_this = static_cast<QQuickStateGroup *>(list->object);
-    return _this->d_func()->transitions.count();
+    return _this->d_func()->transitions.size();
 }
 
 QQuickTransition *QQuickStateGroupPrivate::at_transition(QQmlListProperty<QQuickTransition> *list, qsizetype index)
@@ -333,8 +299,8 @@ void QQuickStateGroup::componentComplete()
     d->componentComplete = true;
 
     QVarLengthArray<QString, 4> names;
-    names.reserve(d->states.count());
-    for (int ii = 0; ii < d->states.count(); ++ii) {
+    names.reserve(d->states.size());
+    for (int ii = 0; ii < d->states.size(); ++ii) {
         QQuickState *state = d->states.at(ii);
         if (!state->isNamed())
             state->setName(QLatin1String("anonymousState") + QString::number(++d->unnamedCount));
@@ -372,21 +338,31 @@ bool QQuickStateGroupPrivate::updateAutoState()
         return false;
 
     bool revert = false;
-    for (int ii = 0; ii < states.count(); ++ii) {
+    for (int ii = 0; ii < states.size(); ++ii) {
         QQuickState *state = states.at(ii);
         if (state->isWhenKnown()) {
             if (state->isNamed()) {
                 bool whenValue = state->when();
-                const QQmlProperty whenProp(state, u"when"_qs);
+                const QQmlProperty whenProp(state, u"when"_s);
                 const auto potentialWhenBinding = QQmlAnyBinding::ofProperty(whenProp);
                 Q_ASSERT(!potentialWhenBinding.isUntypedPropertyBinding());
+
                 // if there is a binding, the value in when might not be up-to-date at this point
-                // so we manually reevaluate the binding
-                if (auto abstractBinding = dynamic_cast<QQmlBinding *>( potentialWhenBinding.asAbstractBinding()))
-                    whenValue = abstractBinding->evaluate().toBool();
+                // so we manually re-evaluate the binding
+                QQmlAbstractBinding *abstractBinding = potentialWhenBinding.asAbstractBinding();
+                if (abstractBinding && abstractBinding->kind() == QQmlAbstractBinding::QmlBinding) {
+                    QQmlBinding *binding = static_cast<QQmlBinding *>(abstractBinding);
+                    if (binding->hasValidContext()) {
+                        QVariant evalResult = binding->evaluate();
+                        if (evalResult.metaType() == QMetaType::fromType<QJSValue>())
+                            whenValue = evalResult.value<QJSValue>().toBool();
+                        else
+                            whenValue = evalResult.toBool();
+                    }
+                }
+
                 if (whenValue) {
-                    if (stateChangeDebug())
-                        qWarning() << "Setting auto state due to expression";
+                    qCDebug(lcStates) << "Setting auto state due to expression";
                     if (currentState != state->name()) {
                         q->setState(state->name());
                         return true;
@@ -415,7 +391,7 @@ QQuickTransition *QQuickStateGroupPrivate::findTransition(const QString &from, c
     bool reversed = false;
     bool done = false;
 
-    for (int ii = 0; !done && ii < transitions.count(); ++ii) {
+    for (int ii = 0; !done && ii < transitions.size(); ++ii) {
         QQuickTransition *t = transitions.at(ii);
         if (!t->enabled())
             continue;
@@ -429,10 +405,10 @@ QQuickTransition *QQuickStateGroupPrivate::findTransition(const QString &from, c
             const QString toStateStr = t->toState();
 
             auto fromState = QStringView{fromStateStr}.split(QLatin1Char(','));
-            for (int jj = 0; jj < fromState.count(); ++jj)
+            for (int jj = 0; jj < fromState.size(); ++jj)
                 fromState[jj] = fromState.at(jj).trimmed();
             auto toState = QStringView{toStateStr}.split(QLatin1Char(','));
-            for (int jj = 0; jj < toState.count(); ++jj)
+            for (int jj = 0; jj < toState.size(); ++jj)
                 toState[jj] = toState.at(jj).trimmed();
             if (ii == 1)
                 qSwap(fromState, toState);
@@ -491,16 +467,16 @@ void QQuickStateGroupPrivate::setCurrentStateInternal(const QString &state,
     applyingState = true;
 
     QQuickTransition *transition = ignoreTrans ? nullptr : findTransition(currentState, state);
-    if (stateChangeDebug()) {
-        qWarning() << this << "Changing state.  From" << currentState << ". To" << state;
+    if (lcStates().isDebugEnabled()) {
+        qCDebug(lcStates) << this << "changing state from:" << currentState << "to:" << state;
         if (transition)
-            qWarning() << "   using transition" << transition->fromState()
-                       << transition->toState();
+            qCDebug(lcStates) << "   using transition" << transition->fromState()
+                              << transition->toState();
     }
 
     QQuickState *oldState = nullptr;
     if (!currentState.isEmpty()) {
-        for (int ii = 0; ii < states.count(); ++ii) {
+        for (int ii = 0; ii < states.size(); ++ii) {
             if (states.at(ii)->name() == currentState) {
                 oldState = states.at(ii);
                 break;
@@ -512,7 +488,7 @@ void QQuickStateGroupPrivate::setCurrentStateInternal(const QString &state,
     emit q->stateChanged(currentState);
 
     QQuickState *newState = nullptr;
-    for (int ii = 0; ii < states.count(); ++ii) {
+    for (int ii = 0; ii < states.size(); ++ii) {
         if (states.at(ii)->name() == currentState) {
             newState = states.at(ii);
             break;
@@ -536,7 +512,7 @@ void QQuickStateGroupPrivate::setCurrentStateInternal(const QString &state,
 QQuickState *QQuickStateGroup::findState(const QString &name) const
 {
     Q_D(const QQuickStateGroup);
-    for (int i = 0; i < d->states.count(); ++i) {
+    for (int i = 0; i < d->states.size(); ++i) {
         QQuickState *state = d->states.at(i);
         if (state->name() == name)
             return state;
