@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the Qt Quick Layouts module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qquicklinearlayout_p.h"
 #include "qquickgridlayoutengine_p.h"
@@ -243,7 +207,7 @@ QQuickGridLayoutBase::QQuickGridLayoutBase()
 
 QQuickGridLayoutBase::QQuickGridLayoutBase(QQuickGridLayoutBasePrivate &dd,
                                            Qt::Orientation orientation,
-                                           QQuickItem *parent /*= 0*/)
+                                           QQuickItem *parent /*= nullptr */)
     : QQuickLayout(dd, parent)
 {
     Q_D(QQuickGridLayoutBase);
@@ -270,7 +234,6 @@ void QQuickGridLayoutBase::setOrientation(Qt::Orientation orientation)
 QSizeF QQuickGridLayoutBase::sizeHint(Qt::SizeHint whichSizeHint) const
 {
     Q_D(const QQuickGridLayoutBase);
-    ensureLayoutItemsUpdated();
     return d->engine.sizeHint(whichSizeHint, QSizeF(), d->styleInfo);
 }
 
@@ -318,6 +281,7 @@ void QQuickGridLayoutBase::setAlignment(QQuickItem *item, Qt::Alignment alignmen
 {
     Q_D(QQuickGridLayoutBase);
     d->engine.setAlignment(item, alignment);
+    maybeSubscribeToBaseLineOffsetChanges(item);
 }
 
 QQuickGridLayoutBase::~QQuickGridLayoutBase()
@@ -346,7 +310,7 @@ void QQuickGridLayoutBase::componentComplete()
        need to call invalidate() in advance
     */
     invalidate();
-    ensureLayoutItemsUpdated();
+    ensureLayoutItemsUpdated(QQuickLayout::ApplySizeHints);
 
     QQuickItem *par = parentItem();
     if (qobject_cast<QQuickLayout*>(par))
@@ -422,10 +386,6 @@ void QQuickGridLayoutBase::updateLayoutItems()
     Q_D(QQuickGridLayoutBase);
     if (!isReady())
         return;
-    if (d->m_rearranging) {
-        d->m_updateAfterRearrange = true;
-        return;
-    }
 
     qCDebug(lcQuickLayouts) << "QQuickGridLayoutBase::updateLayoutItems ENTERING" << this;
     d->engine.deleteItems();
@@ -436,16 +396,12 @@ void QQuickGridLayoutBase::updateLayoutItems()
 QQuickItem *QQuickGridLayoutBase::itemAt(int index) const
 {
     Q_D(const QQuickGridLayoutBase);
-    qCDebug(lcQuickLayouts).nospace() << "QQuickGridLayoutBase::itemAt(" << index << ")";
-    ensureLayoutItemsUpdated();
-    qCDebug(lcQuickLayouts).nospace() << "QQuickGridLayoutBase::itemAt(" << index << ") LEAVING";
     return static_cast<QQuickGridLayoutItem*>(d->engine.itemAt(index))->layoutItem();
 }
 
 int QQuickGridLayoutBase::itemCount() const
 {
     Q_D(const QQuickGridLayoutBase);
-    ensureLayoutItemsUpdated();
     return d->engine.itemCount();
 }
 
@@ -497,8 +453,6 @@ void QQuickGridLayoutBase::rearrange(const QSizeF &size)
         return;
     }
 
-    ensureLayoutItemsUpdated();
-
     d->m_rearranging = true;
     qCDebug(lcQuickLayouts) << objectName() << "QQuickGridLayoutBase::rearrange()" << size;
     Qt::LayoutDirection visualDir = effectiveLayoutDirection();
@@ -517,14 +471,9 @@ void QQuickGridLayoutBase::rearrange(const QSizeF &size)
     d->engine.setGeometries(QRectF(QPointF(0,0), size), d->styleInfo);
     d->m_rearranging = false;
 
-    for (QQuickItem *invalid : qAsConst(d->m_invalidateAfterRearrange))
+    for (QQuickItem *invalid : std::as_const(d->m_invalidateAfterRearrange))
         invalidate(invalid);
     d->m_invalidateAfterRearrange.clear();
-
-    if (d->m_updateAfterRearrange) {
-        ensureLayoutItemsUpdated();
-        d->m_updateAfterRearrange = false;
-    }
 }
 
 /**********************************
@@ -532,7 +481,7 @@ void QQuickGridLayoutBase::rearrange(const QSizeF &size)
  ** QQuickGridLayout
  **
  **/
-QQuickGridLayout::QQuickGridLayout(QQuickItem *parent /* = 0*/)
+QQuickGridLayout::QQuickGridLayout(QQuickItem *parent /* = nullptr*/)
     : QQuickGridLayoutBase(*new QQuickGridLayoutPrivate, Qt::Horizontal, parent)
 {
 }
@@ -684,15 +633,13 @@ void QQuickGridLayout::insertLayoutItems()
     if (flowBound < 0)
         flowBound = std::numeric_limits<int>::max();
 
-    QSizeF sizeHints[Qt::NSizeHints];
     const auto items = childItems();
     for (QQuickItem *child : items) {
         checkAnchors(child);
-        QQuickLayoutAttached *info = nullptr;
-
         // Will skip all items with effective maximum width/height == 0
-        if (shouldIgnoreItem(child, info, sizeHints))
+        if (shouldIgnoreItem(child))
             continue;
+        QQuickLayoutAttached *info = attachedLayoutObject(child, false);
 
         Qt::Alignment alignment;
         int row = -1;
@@ -785,8 +732,6 @@ void QQuickGridLayout::insertLayoutItems()
         column = nextColumn;
         row = nextRow;
         QQuickGridLayoutItem *layoutItem = new QQuickGridLayoutItem(child, row, column, rowSpan, columnSpan, alignment);
-        layoutItem->setCachedSizeHints(sizeHints);
-
         d->engine.insertItem(layoutItem, -1);
     }
 }
@@ -797,7 +742,7 @@ void QQuickGridLayout::insertLayoutItems()
  **
  **/
 QQuickLinearLayout::QQuickLinearLayout(Qt::Orientation orientation,
-                                        QQuickItem *parent /*= 0*/)
+                                        QQuickItem *parent /*= nullptr*/)
     : QQuickGridLayoutBase(*new QQuickLinearLayoutPrivate, orientation, parent)
 {
 }
@@ -871,16 +816,15 @@ void QQuickLinearLayout::setSpacing(qreal space)
 void QQuickLinearLayout::insertLayoutItems()
 {
     Q_D(QQuickLinearLayout);
-    QSizeF sizeHints[Qt::NSizeHints];
     const auto items = childItems();
     for (QQuickItem *child : items) {
         Q_ASSERT(child);
         checkAnchors(child);
-        QQuickLayoutAttached *info = nullptr;
 
         // Will skip all items with effective maximum width/height == 0
-        if (shouldIgnoreItem(child, info, sizeHints))
+        if (shouldIgnoreItem(child))
             continue;
+        QQuickLayoutAttached *info = attachedLayoutObject(child, false);
 
         Qt::Alignment alignment;
         if (info)
@@ -894,7 +838,6 @@ void QQuickLinearLayout::insertLayoutItems()
         if (d->orientation == Qt::Vertical)
             qSwap(gridRow, gridColumn);
         QQuickGridLayoutItem *layoutItem = new QQuickGridLayoutItem(child, gridRow, gridColumn, 1, 1, alignment);
-        layoutItem->setCachedSizeHints(sizeHints);
         d->engine.insertItem(layoutItem, index);
     }
 }
