@@ -1,38 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2020 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
-**
-** This file is part of the Qt Quick Templates 2 module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL3$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPLv3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or later as published by the Free
-** Software Foundation and appearing in the file LICENSE.GPL included in
-** the packaging of this file. Please review the following information to
-** ensure the GNU General Public License version 2.0 requirements will be
-** met: http://www.gnu.org/licenses/gpl-2.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2020 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qquickcontainer_p.h"
 #include "qquickcontainer_p_p.h"
@@ -329,7 +296,7 @@ void QQuickContainerPrivate::reorderItems()
     QList<QQuickItem *> siblings = effectiveContentItem(contentItem)->childItems();
 
     int to = 0;
-    for (int i = 0; i < siblings.count(); ++i) {
+    for (int i = 0; i < siblings.size(); ++i) {
         QQuickItem* sibling = siblings.at(i);
         if (QQuickItemPrivate::get(sibling)->isTransparentForPositioner())
             continue;
@@ -395,7 +362,7 @@ void QQuickContainerPrivate::contentData_append(QQmlListProperty<QObject> *prop,
 qsizetype QQuickContainerPrivate::contentData_count(QQmlListProperty<QObject> *prop)
 {
     QQuickContainer *q = static_cast<QQuickContainer *>(prop->object);
-    return QQuickContainerPrivate::get(q)->contentData.count();
+    return QQuickContainerPrivate::get(q)->contentData.size();
 }
 
 QObject *QQuickContainerPrivate::contentData_at(QQmlListProperty<QObject> *prop, qsizetype index)
@@ -619,7 +586,7 @@ QVariant QQuickContainer::contentModel() const
 }
 
 /*!
-    \qmlproperty list<Object> QtQuick.Controls::Container::contentData
+    \qmlproperty list<QtObject> QtQuick.Controls::Container::contentData
     \qmldefault
 
     This property holds the list of content data.
