@@ -1,38 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2017 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
-**
-** This file is part of the Qt Quick Templates 2 module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL3$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPLv3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or later as published by the Free
-** Software Foundation and appearing in the file LICENSE.GPL included in
-** the packaging of this file. Please review the following information to
-** ensure the GNU General Public License version 2.0 requirements will be
-** met: http://www.gnu.org/licenses/gpl-2.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2017 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qquickcontrol_p.h"
 #include "qquickcontrol_p_p.h"
@@ -179,12 +146,6 @@ bool QQuickControlPrivate::acceptTouch(const QTouchEvent::TouchPoint &point)
         return true;
     }
 
-    // If the control is on a Flickable that has a pressDelay, then the press is never
-    // sent as a touch event, therefore we need to check for this case.
-    if (touchId == -1 && pressWasTouch && point.state() == QEventPoint::Released &&
-        point.position() == previousPressPos) {
-        return true;
-    }
     return false;
 }
 #endif
@@ -197,14 +158,17 @@ static void setActiveFocus(QQuickControl *control, Qt::FocusReason reason)
     control->forceActiveFocus(reason);
 }
 
-void QQuickControlPrivate::handlePress(const QPointF &)
+bool QQuickControlPrivate::handlePress(const QPointF &, ulong)
 {
     Q_Q(QQuickControl);
-    if ((focusPolicy & Qt::ClickFocus) == Qt::ClickFocus && !QGuiApplication::styleHints()->setFocusOnTouchRelease())
+    if ((focusPolicy & Qt::ClickFocus) == Qt::ClickFocus && !QGuiApplication::styleHints()->setFocusOnTouchRelease()) {
         setActiveFocus(q, Qt::MouseFocusReason);
+        return true;
+    }
+    return true;
 }
 
-void QQuickControlPrivate::handleMove(const QPointF &point)
+bool QQuickControlPrivate::handleMove(const QPointF &point, ulong)
 {
 #if QT_CONFIG(quicktemplates2_hover)
     Q_Q(QQuickControl);
@@ -212,16 +176,19 @@ void QQuickControlPrivate::handleMove(const QPointF &point)
 #else
     Q_UNUSED(point);
 #endif
+    return true;
 }
 
-void QQuickControlPrivate::handleRelease(const QPointF &)
+bool QQuickControlPrivate::handleRelease(const QPointF &, ulong)
 {
     Q_Q(QQuickControl);
-    if ((focusPolicy & Qt::ClickFocus) == Qt::ClickFocus && QGuiApplication::styleHints()->setFocusOnTouchRelease())
+    bool accepted = true;
+    if ((focusPolicy & Qt::ClickFocus) == Qt::ClickFocus && QGuiApplication::styleHints()->setFocusOnTouchRelease()) {
         setActiveFocus(q, Qt::MouseFocusReason);
+        accepted = true;
+    }
     touchId = -1;
-    pressWasTouch = false;
-    previousPressPos = QPointF();
+    return accepted;
 }
 
 void QQuickControlPrivate::handleUngrab()
@@ -381,15 +348,26 @@ void QQuickControlPrivate::resizeBackground()
     resizingBackground = true;
 
     QQuickItemPrivate *p = QQuickItemPrivate::get(background);
+    bool changeWidth = false;
+    bool changeHeight = false;
     if (((!p->widthValid() || !extra.isAllocated() || !extra->hasBackgroundWidth) && qFuzzyIsNull(background->x()))
             || (extra.isAllocated() && (extra->hasLeftInset || extra->hasRightInset))) {
         background->setX(getLeftInset());
-        background->setWidth(width - getLeftInset() - getRightInset());
+        changeWidth = !p->width.hasBinding();
     }
     if (((!p->heightValid() || !extra.isAllocated() || !extra->hasBackgroundHeight) && qFuzzyIsNull(background->y()))
             || (extra.isAllocated() && (extra->hasTopInset || extra->hasBottomInset))) {
         background->setY(getTopInset());
-        background->setHeight(height - getTopInset() - getBottomInset());
+        changeHeight = !p->height.hasBinding();
+    }
+    if (changeHeight || changeWidth) {
+        auto newWidth = changeWidth ?
+            width.valueBypassingBindings() - getLeftInset() - getRightInset() :
+            p->width.valueBypassingBindings();
+        auto newHeight = changeHeight ?
+            height.valueBypassingBindings() - getTopInset() - getBottomInset() :
+            p->height.valueBypassingBindings();
+        background->setSize({newWidth, newHeight});
     }
 
     resizingBackground = false;
@@ -522,7 +500,7 @@ QQuickAccessibleAttached *QQuickControlPrivate::accessibleAttached(const QObject
 /*!
     \internal
 
-    Returns the font that the control w inherits from its ancestors and
+    Returns the font that the control \a item inherits from its ancestors and
     QGuiApplication::font.
 */
 QFont QQuickControlPrivate::parentFont(const QQuickItem *item)
@@ -530,13 +508,13 @@ QFont QQuickControlPrivate::parentFont(const QQuickItem *item)
     QQuickItem *p = item->parentItem();
     while (p) {
         if (QQuickControl *control = qobject_cast<QQuickControl *>(p))
-            return control->font();
+            return QQuickControlPrivate::get(control)->resolvedFont;
         else if (QQuickLabel *label = qobject_cast<QQuickLabel *>(p))
-            return label->font();
+            return label->QQuickText::font();
         else if (QQuickTextField *textField = qobject_cast<QQuickTextField *>(p))
-            return textField->font();
+            return textField->QQuickTextInput::font();
         else if (QQuickTextArea *textArea = qobject_cast<QQuickTextArea *>(p))
-            return textArea->font();
+            return textArea->QQuickTextEdit::font();
 
         p = p->parentItem();
     }
@@ -757,6 +735,13 @@ void QQuickControlPrivate::executeBackground(bool complete)
         quickCompleteDeferred(q, backgroundName(), background);
 }
 
+/*
+    \internal
+
+    Hides an item that was replaced by a newer one, rather than
+    deleting it, as the item is typically created in QML and hence
+    we don't own it.
+*/
 void QQuickControlPrivate::hideOldItem(QQuickItem *item)
 {
     if (!item)
@@ -775,6 +760,29 @@ void QQuickControlPrivate::hideOldItem(QQuickItem *item)
 #endif
 }
 
+/*
+    \internal
+
+    Named "unhide" because it's used for cases where an item
+    that was previously hidden by \l hideOldItem() wants to be
+    shown by a control again, such as a ScrollBar in ScrollView.
+*/
+void QQuickControlPrivate::unhideOldItem(QQuickControl *control, QQuickItem *item)
+{
+    Q_ASSERT(item);
+    qCDebug(lcItemManagement) << "unhiding old item" << item;
+
+    item->setVisible(true);
+    item->setParentItem(control);
+
+#if QT_CONFIG(accessibility)
+    // Add the item back in to the accessibility tree.
+    QQuickAccessibleAttached *accessible = accessibleAttached(item);
+    if (accessible)
+        accessible->setIgnored(false);
+#endif
+}
+
 void QQuickControlPrivate::updateBaselineOffset()
 {
     Q_Q(QQuickControl);
@@ -968,12 +976,15 @@ void QQuickControl::itemChange(QQuickItem::ItemChange change, const QQuickItem::
     \endcode
 
     For the full list of available font properties, see the
-    \l [QtQuick]{font}{font QML Basic Type} documentation.
+    \l [QtQuick]{font}{font QML Value Type} documentation.
 */
 QFont QQuickControl::font() const
 {
     Q_D(const QQuickControl);
-    return d->resolvedFont;
+    QFont font = d->resolvedFont;
+    // The resolveMask should inherit from the requestedFont
+    font.setResolveMask(d->extra.value().requestedFont.resolveMask());
+    return font;
 }
 
 void QQuickControl::setFont(const QFont &font)
@@ -1574,7 +1585,6 @@ void QQuickControl::setBackground(QQuickItem *background)
         text: qsTr("Button")
         contentItem: Label {
             text: control.text
-            font: control.font
             verticalAlignment: Text.AlignVCenter
         }
     }
@@ -1954,47 +1964,39 @@ void QQuickControl::hoverEnterEvent(QHoverEvent *event)
 {
     Q_D(QQuickControl);
     setHovered(d->hoverEnabled);
-    event->setAccepted(d->hoverEnabled);
+    event->ignore();
 }
 
 void QQuickControl::hoverMoveEvent(QHoverEvent *event)
 {
     Q_D(QQuickControl);
     setHovered(d->hoverEnabled && contains(event->position()));
-    event->setAccepted(d->hoverEnabled);
+    event->ignore();
 }
 
 void QQuickControl::hoverLeaveEvent(QHoverEvent *event)
 {
-    Q_D(QQuickControl);
     setHovered(false);
-    event->setAccepted(d->hoverEnabled);
+    event->ignore();
 }
 #endif
 
 void QQuickControl::mousePressEvent(QMouseEvent *event)
 {
     Q_D(QQuickControl);
-    d->handlePress(event->position());
-    if (event->source() == Qt::MouseEventSynthesizedByQt) {
-        d->pressWasTouch = true;
-        d->previousPressPos = event->position();
-    }
-    event->accept();
+    event->setAccepted(d->handlePress(event->position(), event->timestamp()));
 }
 
 void QQuickControl::mouseMoveEvent(QMouseEvent *event)
 {
     Q_D(QQuickControl);
-    d->handleMove(event->position());
-    event->accept();
+    event->setAccepted(d->handleMove(event->position(), event->timestamp()));
 }
 
 void QQuickControl::mouseReleaseEvent(QMouseEvent *event)
 {
     Q_D(QQuickControl);
-    d->handleRelease(event->position());
-    event->accept();
+    event->setAccepted(d->handleRelease(event->position(), event->timestamp()));
 }
 
 void QQuickControl::mouseUngrabEvent()
@@ -2017,13 +2019,13 @@ void QQuickControl::touchEvent(QTouchEvent *event)
 
             switch (point.state()) {
             case QEventPoint::Pressed:
-                d->handlePress(point.position());
+                d->handlePress(point.position(), event->timestamp());
                 break;
             case QEventPoint::Updated:
-                d->handleMove(point.position());
+                d->handleMove(point.position(), event->timestamp());
                 break;
             case QEventPoint::Released:
-                d->handleRelease(point.position());
+                d->handleRelease(point.position(), event->timestamp());
                 break;
             default:
                 break;
@@ -2187,3 +2189,5 @@ bool QQuickControl::setAccessibleProperty(const char *propertyName, const QVaria
 }
 
 QT_END_NAMESPACE
+
+#include "moc_qquickcontrol_p.cpp"
