@@ -1,38 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2017 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
-**
-** This file is part of the Qt Quick Templates 2 module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL3$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPLv3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or later as published by the Free
-** Software Foundation and appearing in the file LICENSE.GPL included in
-** the packaging of this file. Please review the following information to
-** ensure the GNU General Public License version 2.0 requirements will be
-** met: http://www.gnu.org/licenses/gpl-2.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2017 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qquickdialogbuttonbox_p.h"
 #include "qquickdialogbuttonbox_p_p.h"
@@ -315,7 +282,7 @@ void QQuickDialogButtonBoxPrivate::updateLayout()
 
     std::stable_sort(buttons.begin(), buttons.end(), ButtonLayout(static_cast<QPlatformDialogHelper::ButtonLayout>(buttonLayout)));
 
-    for (int i = 0; i < buttons.count() - 1; ++i)
+    for (int i = 0; i < buttons.size() - 1; ++i)
         q->insertItem(i, buttons.at(i));
 }
 
@@ -476,6 +443,14 @@ void QQuickDialogButtonBoxPrivate::updateLanguage()
     }
 }
 
+QPlatformDialogHelper::StandardButton QQuickDialogButtonBoxPrivate::standardButton(QQuickAbstractButton *button) const {
+    QQuickDialogButtonBoxAttached *attached = qobject_cast<QQuickDialogButtonBoxAttached *>(qmlAttachedPropertiesObject<QQuickDialogButtonBox>(button, false));
+    if (attached)
+        return QQuickDialogButtonBoxAttachedPrivate::get(attached)->standardButton;
+    else
+        return QPlatformDialogHelper::NoButton;
+}
+
 QQuickDialogButtonBox::QQuickDialogButtonBox(QQuickItem *parent)
     : QQuickContainer(*(new QQuickDialogButtonBoxPrivate), parent)
 {
@@ -486,6 +461,13 @@ QQuickDialogButtonBox::QQuickDialogButtonBox(QQuickItem *parent)
 
 QQuickDialogButtonBox::~QQuickDialogButtonBox()
 {
+    Q_D(QQuickDialogButtonBox);
+    // QQuickContainerPrivate does call this, but as our type information has already been
+    // destroyed by that point (since this destructor has already run), it won't call our
+    // implementation. So, we need to make sure our implementation is called. If we don't do this,
+    // the listener we installed on the contentItem won't get removed, possibly resulting in
+    // heap-use-after-frees.
+    contentItemChange(nullptr, d->contentItem);
 }
 
 /*!
@@ -857,3 +839,5 @@ void QQuickDialogButtonBoxAttached::setButtonRole(QPlatformDialogHelper::ButtonR
 }
 
 QT_END_NAMESPACE
+
+#include "moc_qquickdialogbuttonbox_p.cpp"
