@@ -1,38 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2021 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
-**
-** This file is part of the Qt Quick Templates 2 module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL3$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPLv3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or later as published by the Free
-** Software Foundation and appearing in the file LICENSE.GPL included in
-** the packaging of this file. Please review the following information to
-** ensure the GNU General Public License version 2.0 requirements will be
-** met: http://www.gnu.org/licenses/gpl-2.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2021 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qquickselectionrectangle_p.h"
 #include "qquickselectionrectangle_p_p.h"
@@ -166,7 +133,7 @@ QQuickSelectionRectanglePrivate::QQuickSelectionRectanglePrivate()
     m_dragHandler->setTarget(nullptr);
 
     QObject::connect(&m_scrollTimer, &QTimer::timeout, [&]{
-        if (m_topLeftHandle && m_draggedHandle == m_topLeftHandle)
+        if (m_topLeftHandle && m_draggedHandle == m_topLeftHandle.data())
             m_selectable->setSelectionStartPos(m_scrollToPoint);
         else
             m_selectable->setSelectionEndPos(m_scrollToPoint);
@@ -177,12 +144,11 @@ QQuickSelectionRectanglePrivate::QQuickSelectionRectanglePrivate()
         m_scrollSpeed = QSizeF(qAbs(dist.width() * 0.007), qAbs(dist.height() * 0.007));
     });
 
-    QObject::connect(m_tapHandler, &QQuickTapHandler::tapped, [=](QEventPoint) {
-        m_selectable->clearSelection();
+    QObject::connect(m_tapHandler, &QQuickTapHandler::tapped, [this] {
         updateActiveState(false);
     });
 
-    QObject::connect(m_tapHandler, &QQuickTapHandler::longPressed, [=]() {
+    QObject::connect(m_tapHandler, &QQuickTapHandler::longPressed, [this]() {
         if (handleUnderPos(m_tapHandler->point().pressPosition()) != nullptr) {
             // Don't allow press'n'hold to start a new
             // selection if it started on top of a handle.
@@ -206,12 +172,13 @@ QQuickSelectionRectanglePrivate::QQuickSelectionRectanglePrivate()
         updateActiveState(true);
     });
 
-    QObject::connect(m_dragHandler, &QQuickDragHandler::activeChanged, [=]() {
-        const QPointF pos = m_dragHandler->centroid().position();
+    QObject::connect(m_dragHandler, &QQuickDragHandler::activeChanged, [this]() {
+        const QPointF startPos = m_dragHandler->centroid().pressPosition();
+        const QPointF dragPos = m_dragHandler->centroid().position();
         if (m_dragHandler->active()) {
             m_selectable->clearSelection();
-            m_selectable->setSelectionStartPos(pos);
-            m_selectable->setSelectionEndPos(pos);
+            m_selectable->setSelectionStartPos(startPos);
+            m_selectable->setSelectionEndPos(dragPos);
             m_draggedHandle = nullptr;
             updateHandles();
             updateActiveState(true);
@@ -223,7 +190,7 @@ QQuickSelectionRectanglePrivate::QQuickSelectionRectanglePrivate()
         }
     });
 
-    QObject::connect(m_dragHandler, &QQuickDragHandler::centroidChanged, [=]() {
+    QObject::connect(m_dragHandler, &QQuickDragHandler::centroidChanged, [this]() {
         if (!m_dragging)
             return;
         const QPointF pos = m_dragHandler->centroid().position();
@@ -250,13 +217,13 @@ QQuickItem *QQuickSelectionRectanglePrivate::handleUnderPos(const QPointF &pos)
     if (m_topLeftHandle) {
         const QPointF localPos = m_topLeftHandle->mapFromItem(handlerTarget, pos);
         if (m_topLeftHandle->contains(localPos))
-            return m_topLeftHandle;
+            return m_topLeftHandle.data();
     }
 
     if (m_bottomRightHandle) {
         const QPointF localPos = m_bottomRightHandle->mapFromItem(handlerTarget, pos);
         if (m_bottomRightHandle->contains(localPos))
-            return m_bottomRightHandle;
+            return m_bottomRightHandle.data();
     }
 
     return nullptr;
@@ -300,16 +267,14 @@ QQuickItem *QQuickSelectionRectanglePrivate::createHandle(QQmlComponent *delegat
     // Add pointer handlers to it
     QQuickDragHandler *dragHandler = new QQuickDragHandler();
     dragHandler->setTarget(nullptr);
-    dragHandler->setParent(handleItem);
-    QQuickItemPrivate::get(handleItem)->addPointerHandler(dragHandler);
+    dragHandler->setParentItem(handleItem);
 
     QQuickHoverHandler *hoverHandler = new QQuickHoverHandler();
     hoverHandler->setTarget(nullptr);
-    hoverHandler->setParent(handleItem);
+    hoverHandler->setParentItem(handleItem);
     hoverHandler->setCursorShape(Qt::SizeFDiagCursor);
-    QQuickItemPrivate::get(handleItem)->addPointerHandler(hoverHandler);
 
-    QObject::connect(dragHandler, &QQuickDragHandler::activeChanged, [=]() {
+    QObject::connect(dragHandler, &QQuickDragHandler::activeChanged, [this, corner, handleItem, dragHandler]() {
         if (dragHandler->active()) {
             const QPointF localPos = dragHandler->centroid().position();
             const QPointF pos = handleItem->mapToItem(handleItem->parentItem(), localPos);
@@ -330,7 +295,7 @@ QQuickItem *QQuickSelectionRectanglePrivate::createHandle(QQmlComponent *delegat
         }
     });
 
-    QObject::connect(dragHandler, &QQuickDragHandler::centroidChanged, [=]() {
+    QObject::connect(dragHandler, &QQuickDragHandler::centroidChanged, [this, corner, handleItem, dragHandler]() {
         if (!m_dragging)
             return;
 
@@ -356,10 +321,10 @@ void QQuickSelectionRectanglePrivate::updateHandles()
     const QRectF rect = m_selectable->selectionRectangle().normalized();
 
     if (!m_topLeftHandle && m_topLeftHandleDelegate)
-        m_topLeftHandle = createHandle(m_topLeftHandleDelegate, Qt::TopLeftCorner);
+        m_topLeftHandle.reset(createHandle(m_topLeftHandleDelegate, Qt::TopLeftCorner));
 
     if (!m_bottomRightHandle && m_bottomRightHandleDelegate)
-        m_bottomRightHandle = createHandle(m_bottomRightHandleDelegate, Qt::BottomRightCorner);
+        m_bottomRightHandle.reset(createHandle(m_bottomRightHandleDelegate, Qt::BottomRightCorner));
 
     if (m_topLeftHandle) {
         m_topLeftHandle->setX(rect.x() - (m_topLeftHandle->width() / 2));
@@ -422,6 +387,8 @@ QQuickSelectionRectangle::QQuickSelectionRectangle(QQuickItem *parent)
     : QQuickControl(*(new QQuickSelectionRectanglePrivate), parent)
 {
     Q_D(QQuickSelectionRectangle);
+    d->m_tapHandler->setParent(this);
+    d->m_dragHandler->setParent(this);
 
     QObject::connect(this, &QQuickItem::enabledChanged, [=]() {
         d->m_scrollTimer.stop();
@@ -444,8 +411,8 @@ void QQuickSelectionRectangle::setTarget(QQuickItem *target)
 
     if (d->m_selectable) {
         d->m_scrollTimer.stop();
-        d->m_tapHandler->setParent(nullptr);
-        d->m_dragHandler->setParent(nullptr);
+        d->m_tapHandler->setParent(this);
+        d->m_dragHandler->setParent(this);
         d->m_target->disconnect(this);
     }
 
@@ -460,10 +427,8 @@ void QQuickSelectionRectangle::setTarget(QQuickItem *target)
 
     if (d->m_selectable) {
         const auto handlerTarget = d->m_selectable->selectionPointerHandlerTarget();
-        d->m_dragHandler->setParent(handlerTarget);
-        d->m_tapHandler->setParent(handlerTarget);
-        QQuickItemPrivate::get(handlerTarget)->addPointerHandler(d->m_tapHandler);
-        QQuickItemPrivate::get(handlerTarget)->addPointerHandler(d->m_dragHandler);
+        d->m_dragHandler->setParentItem(handlerTarget);
+        d->m_tapHandler->setParentItem(handlerTarget);
         d->connectToTarget();
         d->updateSelectionMode();
     }
@@ -569,3 +534,5 @@ void QQuickSelectionRectangleAttached::setDragging(bool dragging)
 }
 
 QT_END_NAMESPACE
+
+#include "moc_qquickselectionrectangle_p.cpp"
