@@ -3,10 +3,10 @@
 
 /*---
 esid: sec-array.prototype.reduce
-es5id: 15.4.4.21-5-9
+es5id: 15.4.4.21-3-9
 description: >
-    Array.prototype.reduce - 'initialValue' is returned if 'len' is 0
-    and 'initialValue' is present
+    Array.prototype.reduce - value of 'length' is a number (value is
+    -Infinity)
 ---*/
 
 var accessed = false;
@@ -15,5 +15,10 @@ function callbackfn(prevVal, curVal, idx, obj) {
   accessed = true;
 }
 
-assert.sameValue([].reduce(callbackfn, 3), 3, '[].reduce(callbackfn, 3)');
+var obj = {
+  0: 9,
+  length: -Infinity
+};
+
+assert.sameValue(Array.prototype.reduce.call(obj, callbackfn, 1), 1, 'Array.prototype.reduce.call(obj, callbackfn, 1)');
 assert.sameValue(accessed, false, 'accessed');
