@@ -2,25 +2,24 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 15.3.5.4_2-56gs
+es5id: 15.3.5.4_2-43gs
 description: >
     Strict mode - checking access to strict function caller from
-    non-strict function (strict function declaration called by
-    non-strict function declaration)
+    non-strict function (FunctionExpression with a strict directive
+    prologue defined within a FunctionExpression)
 flags: [noStrict]
 ---*/
 
-function f() {
-  "use strict";
-  gNonStrict();
-};
-
-function foo() {
+var f1 = function() {
+  var f = function() {
+    "use strict";
+    gNonStrict();
+  }
   return f();
 }
 
 assert.throws(TypeError, function() {
-  foo();
+  f1();
 });
 
 function gNonStrict() {
