@@ -2,35 +2,31 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 15.2.3.7-6-a-233
+es5id: 15.2.3.7-6-a-235
 description: >
-    Object.defineProperties - 'O' is an Array, 'P' is an array index
-    property, 'P' is data property and 'desc' is data descriptor, and
-    the [[Configurable]] attribute value of 'P' is false, test
-    TypeError is thrown if the [[Writable]] attribute value of 'P' is
-    false and the [[Writable]] field of 'desc' is true.  (15.4.5.1
-    step 4.c)
+    Object.defineProperties - TypeError is thrown if 'O' is an Array,
+    'P' is an array index named property that already exists on 'O' is
+    data property with  [[Configurable]], [[Writable]] false, 'desc'
+    is data descriptor, [[Value]] field of 'desc' is +0, and the
+    [[Value]] attribute value of 'P' is -0  (15.4.5.1 step 4.c)
 includes: [propertyHelper.js]
 ---*/
 
-
 var arr = [];
 
 Object.defineProperty(arr, "1", {
-  configurable: false,
-  writable: false
-
+  value: +0
 });
 
 try {
   Object.defineProperties(arr, {
     "1": {
-      writable: true
+      value: -0
     }
   });
   $ERROR("Expected an exception.");
 } catch (e) {
-  verifyEqualTo(arr, "1", undefined);
+  verifyEqualTo(arr, "1", +0);
 
   verifyNotWritable(arr, "1");
 
