@@ -2,14 +2,14 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 12.2.1-1-s
+es5id: 12.2.1-7-s
 description: >
-    eval - a function declaring a var named 'eval' throws SyntaxError
-    in strict mode
+    eval - a direct eval declaring a var named 'eval' throws
+    SyntaxError in strict mode
 flags: [onlyStrict]
 ---*/
 
 
 assert.throws(SyntaxError, function() {
-    eval('function foo() { var eval; }');
+    eval('var eval;');
 });
