@@ -1,30 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the test suite of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 #include <QtTest/QtTest>
 #include <QtQml/QtQml>
 #include <QtQuickTestUtils/private/qmlutils_p.h>
@@ -77,6 +52,9 @@ private slots:
     void writeProperty_javascriptExpression_data();
     void writeProperty_javascriptExpression();
 
+    void cyclicStringify();
+    void recursiveStringify();
+
 private:
     QByteArray readAsUtf8(const QString &fileName);
     static QJsonValue valueFromJson(const QByteArray &json);
@@ -514,6 +492,43 @@ void tst_qjsonbinding::writeProperty_javascriptExpression()
     QCOMPARE(ret.toString(), expectedJson);
 }
 
+void tst_qjsonbinding::cyclicStringify()
+{
+    QJSEngine e;
+    const QString program = QStringLiteral(R"(
+        var a = {};
+        a.a = a;
+        JSON.stringify(a);
+    )");
+
+    QJSValue result = e.evaluate(program);
+    QVERIFY(result.isError());
+    QCOMPARE(result.errorType(), QJSValue::TypeError);
+    QVERIFY(result.toString().contains(QLatin1String("Cannot convert circular structure to JSON")));
+}
+
+void tst_qjsonbinding::recursiveStringify()
+{
+    QJSEngine e;
+    const QString program = QStringLiteral(R"(
+        function create() {
+            var a = {}
+            Object.defineProperty(a, "a", {
+                enumerable: true,
+                get: function() { return create(); }
+            });
+            return a;
+        }
+
+        JSON.stringify(create());
+    )");
+
+    QJSValue result = e.evaluate(program);
+    QVERIFY(result.isError());
+    QCOMPARE(result.errorType(), QJSValue::RangeError);
+    QVERIFY(result.toString().contains(QLatin1String("Maximum call stack size exceeded")));
+}
+
 QTEST_MAIN(tst_qjsonbinding)
 
 #include "tst_qjsonbinding.moc"
