@@ -1,30 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the test suite of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include <private/qqmljsengine_p.h>
 #include <private/qqmljsparser_p.h>
@@ -33,7 +8,7 @@
 #include <private/qqmljsast_p.h>
 
 #include <QtQuickTestUtils/private/qmlutils_p.h>
-#include <QtQuickTestUtils/private/qqmljsastdumper_p.h>
+#include <QtQmlDom/private/qqmldomastdumper_p.h>
 
 #include <qtest.h>
 #include <QDir>
@@ -575,7 +550,7 @@ void tst_qqmlparser::typeAnnotations_data()
     QStringList files;
     files << findFiles(QDir(tests));
 
-    for (const QString &file: qAsConst(files))
+    for (const QString &file: std::as_const(files))
         QTest::newRow(qPrintable(file)) << file;
 }
 
@@ -615,7 +590,7 @@ void tst_qqmlparser::disallowedTypeAnnotations_data()
     QStringList files;
     files << findFiles(QDir(tests));
 
-    for (const QString &file: qAsConst(files))
+    for (const QString &file: std::as_const(files))
         QTest::newRow(qPrintable(file)) << file;
 }
 
@@ -704,7 +679,7 @@ void tst_qqmlparser::annotations_data()
     QStringList refFiles;
     refFiles << findFiles(QDir(compare));
 
-    for (const QString &file: qAsConst(files)) {
+    for (const QString &file: std::as_const(files)) {
         auto fileNameStart = file.lastIndexOf(QDir::separator());
         auto fileName = QStringView(file).mid(fileNameStart, file.length()-fileNameStart);
         auto ref=std::find_if(refFiles.constBegin(),refFiles.constEnd(), [fileName](const QString &s){ return s.endsWith(fileName); });
@@ -747,7 +722,9 @@ void tst_qqmlparser::annotations()
         Parser parser2(&engine2);
         QVERIFY(parser2.parse());
 
-        QCOMPARE(AstDumper::diff(parser.ast(), parser2.rootNode(), 3, DumperOptions::NoAnnotations | DumperOptions::NoLocations), QString());
+        using namespace QQmlJS::Dom;
+        QString diff = astNodeDiff(parser.ast(), parser2.rootNode(), 3, AstDumperOption::NoAnnotations | AstDumperOption::NoLocations);
+        QVERIFY2(diff.isEmpty(), qPrintable(diff));
     }
 }
 
