@@ -1,30 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the test suite of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include <QtTest/QtTest>
 #include <QtTest/QSignalSpy>
@@ -54,6 +29,7 @@ public:
 private slots:
     void cleanupTestCase();
     void pinchProperties();
+    void scale_data();
     void scale();
     void scaleThreeFingers();
     void scaleNativeGesture_data();
@@ -99,9 +75,9 @@ void tst_QQuickPinchHandler::pinchProperties()
     QVERIFY(rootItem != nullptr);
     QSignalSpy targetSpy(pinchHandler, SIGNAL(targetChanged()));
     pinchHandler->setTarget(rootItem);
-    QCOMPARE(targetSpy.count(),1);
+    QCOMPARE(targetSpy.size(),1);
     pinchHandler->setTarget(rootItem);
-    QCOMPARE(targetSpy.count(),1);
+    QCOMPARE(targetSpy.size(),1);
 
     // axis
     /*
@@ -154,23 +130,23 @@ void tst_QQuickPinchHandler::pinchProperties()
     QSignalSpy scaleMinSpy(pinchHandler, SIGNAL(minimumScaleChanged()));
     QSignalSpy scaleMaxSpy(pinchHandler, SIGNAL(maximumScaleChanged()));
 
-    QCOMPARE(pinchHandler->minimumScale(), 1.0);
+    QCOMPARE(pinchHandler->minimumScale(), 0.5);
     QCOMPARE(pinchHandler->maximumScale(), 4.0);
 
-    pinchHandler->setMinimumScale(0.5);
+    pinchHandler->setMinimumScale(0.25);
     pinchHandler->setMaximumScale(1.5);
 
-    QCOMPARE(pinchHandler->minimumScale(), 0.5);
+    QCOMPARE(pinchHandler->minimumScale(), 0.25);
     QCOMPARE(pinchHandler->maximumScale(), 1.5);
 
-    QCOMPARE(scaleMinSpy.count(),1);
-    QCOMPARE(scaleMaxSpy.count(),1);
+    QCOMPARE(scaleMinSpy.size(),1);
+    QCOMPARE(scaleMaxSpy.size(),1);
 
-    pinchHandler->setMinimumScale(0.5);
+    pinchHandler->setMinimumScale(0.25);
     pinchHandler->setMaximumScale(1.5);
 
-    QCOMPARE(scaleMinSpy.count(),1);
-    QCOMPARE(scaleMaxSpy.count(),1);
+    QCOMPARE(scaleMinSpy.size(),1);
+    QCOMPARE(scaleMaxSpy.size(),1);
 
     // minimum and maximum rotation properties
     QSignalSpy rotMinSpy(pinchHandler, SIGNAL(minimumRotationChanged()));
@@ -185,110 +161,139 @@ void tst_QQuickPinchHandler::pinchProperties()
     QCOMPARE(pinchHandler->minimumRotation(), -90.0);
     QCOMPARE(pinchHandler->maximumRotation(), 45.0);
 
-    QCOMPARE(rotMinSpy.count(),1);
-    QCOMPARE(rotMaxSpy.count(),1);
+    QCOMPARE(rotMinSpy.size(),1);
+    QCOMPARE(rotMaxSpy.size(),1);
 
     pinchHandler->setMinimumRotation(-90.0);
     pinchHandler->setMaximumRotation(45.0);
 
-    QCOMPARE(rotMinSpy.count(),1);
-    QCOMPARE(rotMaxSpy.count(),1);
+    QCOMPARE(rotMinSpy.size(),1);
+    QCOMPARE(rotMaxSpy.size(),1);
 }
 
-QMutableEventPoint makeTouchPoint(int id, QPoint p, QQuickView *v, QQuickItem *i)
+QEventPoint makeTouchPoint(int id, QPoint p, QQuickView *v, QQuickItem *i)
 {
-    QMutableEventPoint touchPoint(id);
-    touchPoint.setPosition(i->mapFromScene(p));
-    touchPoint.setGlobalPosition(v->mapToGlobal(p));
-    touchPoint.setScenePosition(p);
+    QEventPoint touchPoint(id);
+    QMutableEventPoint::setPosition(touchPoint, i->mapFromScene(p));
+    QMutableEventPoint::setGlobalPosition(touchPoint, v->mapToGlobal(p));
+    QMutableEventPoint::setScenePosition(touchPoint, p);
     return touchPoint;
 }
 
+void tst_QQuickPinchHandler::scale_data()
+{
+    QTest::addColumn<QUrl>("qmlfile");
+    QTest::addColumn<bool>("hasTarget");
+    QTest::newRow("targetModifying") << testFileUrl("pinchproperties.qml") << true;
+    QTest::newRow("nullTarget") << testFileUrl("nullTarget.qml") << false;
+}
+
 void tst_QQuickPinchHandler::scale()
 {
-    QQuickView *window = QQuickViewTestUtils::createView();
-    QScopedPointer<QQuickView> scope(window);
-    window->setSource(testFileUrl("pinchproperties.qml"));
-    window->show();
-    QVERIFY(QTest::qWaitForWindowExposed(window));
-    QVERIFY(window->rootObject() != nullptr);
-    qApp->processEvents();
+    QFETCH(QUrl, qmlfile);
+    QFETCH(bool, hasTarget);
 
-    auto *pinchHandler = static_cast<PinchHandler *>(window->rootObject()->findChild<QQuickPinchHandler*>("pinchHandler"));
-    QVERIFY(pinchHandler != nullptr);
-    QSignalSpy grabChangedSpy(pinchHandler, SIGNAL(grabChanged(QPointingDevice::GrabTransition, QEventPoint)));
-
-    QQuickItem *root = qobject_cast<QQuickItem*>(window->rootObject());
+    QQuickView window;
+    QVERIFY(QQuickTest::showView(window, qmlfile));
+    QQuickItem *root = qobject_cast<QQuickItem*>(window.rootObject());
     QVERIFY(root != nullptr);
-
-    // target
-    QQuickItem *blackRect = window->rootObject()->findChild<QQuickItem*>("blackrect");
+    auto *pinchHandler = static_cast<PinchHandler *>(root->findChild<QQuickPinchHandler*>());
+    QVERIFY(pinchHandler != nullptr);
+    QQuickItem *blackRect = (hasTarget ? pinchHandler->target() : pinchHandler->parentItem());
     QVERIFY(blackRect != nullptr);
+    QSignalSpy grabChangedSpy(pinchHandler, SIGNAL(grabChanged(QPointingDevice::GrabTransition, QEventPoint)));
 
     QPoint p0(80, 80);
     QPoint p1(100, 100);
-    {
-        QTest::QTouchEventSequence pinchSequence = QTest::touchEvent(window, touchscreen);
-        pinchSequence.press(0, p0, window).commit();
-        QQuickTouchUtils::flush(window);
-        // In order for the stationary point to remember its previous position,
-        // we have to reuse the same pinchSequence object.  Otherwise if we let it
-        // be destroyed and then start a new sequence, point 0 will default to being
-        // stationary at 0, 0, and pinchHandler will filter out that touchpoint because
-        // it is outside its bounds.
-        pinchSequence.stationary(0).press(1, p1, window).commit();
-        QQuickTouchUtils::flush(window);
-        QTRY_COMPARE(grabChangedSpy.count(), 1); // passive grab
+    QTest::QTouchEventSequence pinchSequence = QTest::touchEvent(&window, touchscreen);
+    pinchSequence.press(0, p0, &window).commit();
+    QQuickTouchUtils::flush(&window);
+    // In order for the stationary point to remember its previous position,
+    // we have to reuse the same pinchSequence object.  Otherwise if we let it
+    // be destroyed and then start a new sequence, point 0 will default to being
+    // stationary at 0, 0, and pinchHandler will filter out that touchpoint because
+    // it is outside its bounds.
+    pinchSequence.stationary(0).press(1, p1, &window).commit();
+    QQuickTouchUtils::flush(&window);
+    QCOMPARE(grabChangedSpy.size(), 1); // passive grab
 
-        QPoint pd(10, 10);
-        // move one point until PinchHandler activates
-        for (int pi = 0; pi < 10 && !pinchHandler->active(); ++pi) {
-            p1 += pd;
-            pinchSequence.stationary(0).move(1, p1, window).commit();
-            QQuickTouchUtils::flush(window);
-        }
-        QCOMPARE(pinchHandler->active(), true);
-        // grabs occur when the handler becomes active; at that time, QQuickHandlerPoint.sceneGrabPosition should be correct
-        QVERIFY(pinchHandler->firstPoint().sceneGrabPosition() != QPointF());
-        QVERIFY(pinchHandler->lastPoint().sceneGrabPosition() != QPointF());
-        QCOMPARE(pinchHandler->firstPoint().sceneGrabPosition(), pinchHandler->firstPoint().scenePosition());
-        QCOMPARE(pinchHandler->lastPoint().sceneGrabPosition(), pinchHandler->lastPoint().scenePosition());
-        // first point got a passive grab; both points got exclusive grabs
-        QCOMPARE(grabChangedSpy.count(), 3);
-        QLineF line(p0, p1);
-        const qreal startLength = line.length();
+    QPoint pd(10, 10);
+    // move one point until PinchHandler activates
+    for (int pi = 0; pi < 10 && !pinchHandler->active(); ++pi) {
+        p1 += pd;
+        pinchSequence.stationary(0).move(1, p1, &window).commit();
+        QQuickTouchUtils::flush(&window);
+    }
+    QCOMPARE(pinchHandler->active(), true);
+    // grabs occur when the handler becomes active; at that time, QQuickHandlerPoint.sceneGrabPosition should be correct
+    QVERIFY(pinchHandler->firstPoint().sceneGrabPosition() != QPointF());
+    QVERIFY(pinchHandler->lastPoint().sceneGrabPosition() != QPointF());
+    QCOMPARE(pinchHandler->firstPoint().sceneGrabPosition(), pinchHandler->firstPoint().scenePosition());
+    QCOMPARE(pinchHandler->lastPoint().sceneGrabPosition(), pinchHandler->lastPoint().scenePosition());
+    // first point got a passive grab; both points got exclusive grabs
+    QCOMPARE(grabChangedSpy.size(), 3);
+    QLineF line(p0, p1);
+    const qreal startLength = line.length();
 
-        p1+=pd;
-        pinchSequence.stationary(0).move(1, p1, window).commit();
-        QQuickTouchUtils::flush(window);
+    // move the same point even further and observe the change in scale
+    for (int i = 0; i < 2; ++i) {
+        p1 += pd;
+        pinchSequence.stationary(0).move(1, p1, &window).commit();
+        QQuickTouchUtils::flush(&window);
+        if (lcPointerTests().isDebugEnabled()) QTest::qWait(500);
         line.setP2(p1);
-        qreal scale = line.length() / startLength;
-        QVERIFY(qFloatDistance(root->property("scale").toReal(), scale) < 10);
-        QVERIFY(qFloatDistance(blackRect->scale(), scale) < 10);
-
-        p1+=pd;
-        pinchSequence.stationary(0).move(1, p1, window).commit();
-        QQuickTouchUtils::flush(window);
-        line.setP2(p1);
-        scale = line.length() / startLength;
-
-        QVERIFY(qFloatDistance(root->property("scale").toReal(), scale) < 10);
-        QVERIFY(qFloatDistance(blackRect->scale(), scale) < 10);
-
-        QPointF expectedCentroid = p0 + (p1 - p0)/2;
+        qreal expectedScale = line.length() / startLength;
+        QVERIFY(qFloatDistance(root->property("pinchScale").toReal(), expectedScale) < 10);
+        QVERIFY(qFloatDistance(blackRect->scale(), expectedScale) < 10);
+        QCOMPARE(pinchHandler->scale(), root->property("pinchScale").toReal());
+        QCOMPARE(pinchHandler->scale(), pinchHandler->activeScale()); // in sync for the first gesture
+        QPointF expectedCentroid = p0 + (p1 - p0) / 2;
         QCOMPARE(pinchHandler->centroid().scenePosition(), expectedCentroid);
     }
 
-    // scale beyond bound
-    p1 += QPoint(20, 20);
-    {
-        QTest::QTouchEventSequence pinchSequence = QTest::touchEvent(window, touchscreen);
-        pinchSequence.stationary(0).move(1, p1, window).commit();
-        QQuickTouchUtils::flush(window);
-        QCOMPARE(blackRect->scale(), qreal(4));    // qquickpinchhandler does not manipulate scale property
-        pinchSequence.release(0, p0, window).release(1, p1, window).commit();
-        QQuickTouchUtils::flush(window);
+    qreal lastScale = pinchHandler->scale();
+    pinchSequence.release(0, p0, &window).release(1, p1, &window).commit();
+    QQuickTouchUtils::flush(&window);
+    if (lcPointerTests().isDebugEnabled()) QTest::qWait(500);
+    // scale property is persistent after release
+    QCOMPARE(pinchHandler->scale(), lastScale);
+
+    // pinch a second time: scale picks up where we left off
+    p0 = QPoint(80, 80);
+    p1 = QPoint(100, 100);
+    pinchSequence.press(0, p0, &window).press(1, p1, &window).commit();
+    // move one point until PinchHandler activates
+    for (int pi = 0; pi < 10 && !pinchHandler->active(); ++pi) {
+        p1 += pd;
+        pinchSequence.stationary(0).move(1, p1, &window).commit();
+        QQuickTouchUtils::flush(&window);
     }
+    if (lcPointerTests().isDebugEnabled()) QTest::qWait(500);
+    QCOMPARE(pinchHandler->active(), true);
+    QCOMPARE(pinchHandler->scale(), lastScale); // just activated, not scaling further yet
+    for (int i = 0; i < 2; ++i) {
+        lastScale = pinchHandler->scale();
+        p1 += pd;
+        pinchSequence.stationary(0).move(1, p1, &window).commit();
+        QQuickTouchUtils::flush(&window);
+        if (lcPointerTests().isDebugEnabled()) QTest::qWait(500);
+        QCOMPARE_GT(pinchHandler->scale(), lastScale);
+        line.setP2(p1);
+        qreal expectedActiveScale = line.length() / startLength;
+        QVERIFY(qFloatDistance(pinchHandler->activeScale(), expectedActiveScale) < 10);
+        QCOMPARE(pinchHandler->scale(), root->property("pinchScale").toReal());
+        QCOMPARE_NE(pinchHandler->scale(), pinchHandler->activeScale()); // not in sync anymore
+    }
+
+    // scale beyond maximumScale
+    p1 = QPoint(310, 310);
+    pinchSequence.stationary(0).move(1, p1, &window).commit();
+    QQuickTouchUtils::flush(&window);
+    if (lcPointerTests().isDebugEnabled()) QTest::qWait(500);
+    QCOMPARE(blackRect->scale(), qreal(4));
+    QCOMPARE(pinchHandler->scale(), qreal(4)); // limited by maximumScale
+    pinchSequence.release(0, p0, &window).release(1, p1, &window).commit();
+    QQuickTouchUtils::flush(&window);
     QCOMPARE(pinchHandler->active(), false);
 }
 
@@ -373,14 +378,18 @@ void tst_QQuickPinchHandler::scaleThreeFingers()
 void tst_QQuickPinchHandler::scaleNativeGesture_data()
 {
     QTest::addColumn<QString>("qmlfile");
+    QTest::addColumn<qreal>("scale");
 
-    QTest::newRow("just pinch") << "pinchproperties.qml";
-    QTest::newRow("pinch & drag") << "pinchAndDrag.qml";
+    QTest::newRow("just pinch") << "pinchproperties.qml" << 1.1;
+    QTest::newRow("pinch & drag") << "pinchAndDrag.qml" << 1.1;
+    QTest::newRow("bigger than limit") << "pinchproperties.qml" << 5.0;
+    QTest::newRow("smaller than limit") << "pinchproperties.qml" << 0.25;
 }
 
 void tst_QQuickPinchHandler::scaleNativeGesture()
 {
     QFETCH(QString, qmlfile);
+    QFETCH(qreal, scale);
 
     QQuickView *window = QQuickViewTestUtils::createView();
     QScopedPointer<QQuickView> scope(window);
@@ -400,8 +409,8 @@ void tst_QQuickPinchHandler::scaleNativeGesture()
     QPointF targetPos = target->position();
     ulong ts = 1;
 
-    // first pinch: scale it up
-    const qreal expectedScale = 1.1;
+    // first pinch: scale it
+    const qreal expectedScale = qBound(qreal(0.5), scale, qreal(4));
     QPointF pinchPos(75, 75);
     QPointF pinchLocalPos = target->mapFromScene(pinchPos);
     // target position is adjusted in QQuickItemPrivate::adjustedPosForTransform()
@@ -411,11 +420,11 @@ void tst_QQuickPinchHandler::scaleNativeGesture()
     QWindowSystemInterface::handleGestureEvent(window, ts++, touchpad,
                                                Qt::BeginNativeGesture, pinchPos, pinchPos);
     QWindowSystemInterface::handleGestureEventWithRealValue(window, ts++, touchpad,
-                                                            Qt::ZoomNativeGesture, expectedScale - 1, pinchPos, pinchPos);
+                                                            Qt::ZoomNativeGesture, scale - 1, pinchPos, pinchPos);
     QTRY_COMPARE(target->scale(), expectedScale);
     QCOMPARE(pinchHandler->active(), true);
-    QCOMPARE(pinchHandler->centroid().position(), pinchLocalPos);
-    QCOMPARE(pinchHandler->centroid().scenePosition(), pinchPos);
+    QCOMPARE(pinchHandler->centroid().position().toPoint(), pinchLocalPos.toPoint());
+    QCOMPARE(pinchHandler->centroid().scenePosition().toPoint(), pinchPos.toPoint());
     QVERIFY(qAbs(target->position().x() - expectedPos.x()) < 0.001);
     QVERIFY(qAbs(target->position().y() - expectedPos.y()) < 0.001);
     QCOMPARE(pinchHandler->scale(), expectedScale);
@@ -431,9 +440,12 @@ void tst_QQuickPinchHandler::scaleNativeGesture()
     QCOMPARE(pinchHandler->translation(), QVector2D());
     QCOMPARE(pinchHandler->rotation(), 0);
 
-    // second pinch at a different position: scale it down to original size again
+    // second pinch at a different position: scale it back to original size again
+    // but remove the limits first, so that we can scale arbitrarily
+    pinchHandler->setMaximumScale(qInf());
+    pinchHandler->setMinimumScale(-qInf());
     const qreal reverseScale = (1 / expectedScale);
-    pinchPos = QPointF(125, 125);
+    pinchPos = QPointF(110, 110);
     pinchLocalPos = target->mapFromScene(pinchPos);
     QWindowSystemInterface::handleGestureEvent(window, ts++, touchpad,
                                                Qt::BeginNativeGesture, pinchPos, pinchPos);
@@ -441,8 +453,8 @@ void tst_QQuickPinchHandler::scaleNativeGesture()
                                                             Qt::ZoomNativeGesture, reverseScale - 1, pinchPos, pinchPos);
     QTRY_COMPARE(target->scale(), 1);
     QCOMPARE(pinchHandler->active(), true);
-    QCOMPARE(pinchHandler->centroid().position(), pinchLocalPos);
-    QCOMPARE(pinchHandler->centroid().scenePosition(), pinchPos);
+    QCOMPARE(pinchHandler->centroid().position().toPoint(), pinchLocalPos.toPoint());
+    QCOMPARE(pinchHandler->centroid().scenePosition().toPoint(), pinchPos.toPoint());
     QCOMPARE(pinchHandler->scale(), 1);
     QCOMPARE(pinchHandler->activeScale(), reverseScale);
     QWindowSystemInterface::handleGestureEvent(window, ts++, touchpad,
@@ -485,7 +497,7 @@ void tst_QQuickPinchHandler::pan()
         pinchSequence.stationary(0).press(1, p1, window).commit();
         QQuickTouchUtils::flush(window);
         QVERIFY(!root->property("pinchActive").toBool());
-        QCOMPARE(root->property("scale").toReal(), -1.0);
+        QCOMPARE(root->property("pinchScale").toReal(), -1.0);
 
         p0 += QPoint(dragThreshold, 0);
         p1 += QPoint(dragThreshold, 0);
@@ -493,7 +505,7 @@ void tst_QQuickPinchHandler::pan()
         QQuickTouchUtils::flush(window);
         // movement < dragThreshold: pinchHandler not yet active
         QVERIFY(!root->property("pinchActive").toBool());
-        QCOMPARE(root->property("scale").toReal(), -1.0);
+        QCOMPARE(root->property("pinchScale").toReal(), -1.0);
 
         // just above the dragThreshold: pinchHandler starts
         p0 += QPoint(1, 0);
@@ -501,7 +513,7 @@ void tst_QQuickPinchHandler::pan()
         pinchSequence.move(0, p0, window).move(1, p1, window).commit();
         QQuickTouchUtils::flush(window);
         QCOMPARE(pinchHandler->active(), true);
-        QCOMPARE(root->property("scale").toReal(), 1.0);
+        QCOMPARE(root->property("pinchScale").toReal(), 1.0);
 
         // Calculation of the center point is tricky at first:
         // center point of the two touch points in item coordinates:
@@ -668,7 +680,7 @@ void tst_QQuickPinchHandler::retouch()
         pinchSequence.move(0, p0,window).move(1, p1,window).commit();
         QQuickTouchUtils::flush(window);
 
-        QCOMPARE(root->property("scale").toReal(), 1.0);
+        QCOMPARE(root->property("pinchScale").toReal(), 1.0);
         QCOMPARE(pinchHandler->active(), true);
 
         p0 -= delta;
@@ -679,8 +691,8 @@ void tst_QQuickPinchHandler::retouch()
         QCOMPARE(pinchHandler->active(), true);
 
         // accept some slack
-        QVERIFY(withinBounds(1.4, root->property("scale").toReal(), 1.6));
-        QCOMPARE(pinchHandler->centroid().position(), QPointF(40, 40)); // blackrect is at 50,50
+        QVERIFY(withinBounds(1.4, root->property("pinchScale").toReal(), 1.6));
+        QCOMPARE(pinchHandler->centroid().position().toPoint(), QPoint(40, 40)); // blackrect is at 50,50
         QVERIFY(withinBounds(1.4, blackRect->scale(), 1.6));
 
         QCOMPARE(root->property("activeCount").toInt(), 1);
@@ -754,7 +766,7 @@ void tst_QQuickPinchHandler::cancel()
         pinchSequence.move(0, p0,window).move(1, p1,window).commit();
         QQuickTouchUtils::flush(window);
 
-        QCOMPARE(root->property("scale").toReal(), 1.0);
+        QCOMPARE(root->property("pinchScale").toReal(), 1.0);
         QCOMPARE(pinchHandler->active(), true);
 
         p0 -= delta;
@@ -762,8 +774,8 @@ void tst_QQuickPinchHandler::cancel()
         pinchSequence.move(0, p0,window).move(1, p1,window).commit();
         QQuickTouchUtils::flush(window);
 
-        QVERIFY(withinBounds(1.4, root->property("scale").toReal(), 1.6));
-        QCOMPARE(pinchHandler->centroid().position(), QPointF(40, 40)); // blackrect is at 50,50
+        QVERIFY(withinBounds(1.4, root->property("pinchScale").toReal(), 1.6));
+        QCOMPARE(pinchHandler->centroid().position().toPoint(), QPoint(40, 40)); // blackrect is at 50,50
         QVERIFY(withinBounds(1.4, blackRect->scale(), 1.6));
 
         QSKIP("cancel is not supported atm");
@@ -772,8 +784,8 @@ void tst_QQuickPinchHandler::cancel()
         QCoreApplication::sendEvent(window, &cancelEvent);
         QQuickTouchUtils::flush(window);
 
-        QCOMPARE(root->property("scale").toReal(), 1.0);
-        QCOMPARE(root->property("center").toPointF(), QPointF(40, 40)); // blackrect is at 50,50
+        QCOMPARE(root->property("pinchScale").toReal(), 1.0);
+        QCOMPARE(root->property("center").toPoint(), QPoint(40, 40)); // blackrect is at 50,50
         QCOMPARE(blackRect->scale(), 1.0);
         QVERIFY(!root->property("pinchActive").toBool());
     }
