@@ -1,30 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the test suite of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 #include <qtest.h>
 #include <QtQml/qqmlengine.h>
 #include <QtQml/qqmlcomponent.h>
@@ -106,9 +81,9 @@ void addToNewProperty(QObject *object, QObject *newParent, const QByteArray &new
     Q_ASSERT(objectToVariant(object).isValid());
 }
 
-static void removeObjectFromList(const QQmlProperty &property, QObject *objectToBeRemoved, QQmlEngine * engine)
+static void removeObjectFromList(const QQmlProperty &property, QObject *objectToBeRemoved)
 {
-    QQmlListReference listReference(property.object(), property.name().toUtf8(), engine);
+    QQmlListReference listReference(property.object(), property.name().toUtf8());
 
     int count = listReference.count();
 
@@ -312,7 +287,7 @@ void tst_qquickdesignersupport::dynamicProperty()
     QVERIFY(simpleItem);
 
     QQuickDesignerSupportProperties::registerNodeInstanceMetaObject(simpleItem, view->engine());
-    QQuickDesignerSupportProperties::getPropertyCache(simpleItem, view->engine());
+    QQuickDesignerSupportProperties::getPropertyCache(simpleItem);
 
     QQuickDesignerSupportProperties::createNewDynamicProperty(simpleItem, view->engine(), QLatin1String("dynamicProperty"));
 
@@ -344,10 +319,13 @@ void tst_qquickdesignersupport::createComponent()
 
     QVERIFY(rootItem);
 
-    QObject *testComponentObject = QQuickDesignerSupportItems::createComponent(testFileUrl("TestComponent.qml"), view->rootContext());
+    QScopedPointer<QObject> testComponentObject(
+                QQuickDesignerSupportItems::createComponent(
+                    testFileUrl("TestComponent.qml"), view->rootContext()));
     QVERIFY(testComponentObject);
 
-    QVERIFY(QQuickDesignerSupportMetaInfo::isSubclassOf(testComponentObject, "QtQuick/Item"));
+    QVERIFY(QQuickDesignerSupportMetaInfo::isSubclassOf(
+                testComponentObject.data(), "QtQuick/Item"));
 }
 
 void tst_qquickdesignersupport::basicStates()
@@ -366,7 +344,7 @@ void tst_qquickdesignersupport::basicStates()
 
     QVERIFY(stateGroup);
 
-    QCOMPARE(stateGroup->states().count(), 2 );
+    QCOMPARE(stateGroup->states().size(), 2 );
 
     QQuickState *state01 = stateGroup->states().first();
     QQuickState *state02 = stateGroup->states().last();
@@ -412,7 +390,7 @@ void tst_qquickdesignersupport::statesPropertyChanges()
 
     QVERIFY(stateGroup);
 
-    QCOMPARE(stateGroup->states().count(), 2 );
+    QCOMPARE(stateGroup->states().size(), 2 );
 
     QQuickState *state01 = stateGroup->states().first();
     QQuickState *state02 = stateGroup->states().last();
@@ -431,7 +409,7 @@ void tst_qquickdesignersupport::statesPropertyChanges()
 
     QCOMPARE(state01->operationCount(), 1);
 
-    QCOMPARE(statePrivate01->operations.count(), 1);
+    QCOMPARE(statePrivate01->operations.size(), 1);
 
     QQuickStateOperation *propertyChange = statePrivate01->operations.at(0).data();
 
@@ -466,7 +444,7 @@ void tst_qquickdesignersupport::statesPropertyChanges()
     QCOMPARE(rootItem, QQuickDesignerSupportPropertyChanges::targetObject(newPropertyChange));
 
     QCOMPARE(state01->operationCount(), 2);
-    QCOMPARE(statePrivate01->operations.count(), 2);
+    QCOMPARE(statePrivate01->operations.size(), 2);
 
     QCOMPARE(QQuickDesignerSupportPropertyChanges::stateObject(newPropertyChange), state01);
 
@@ -758,7 +736,7 @@ void  tst_qquickdesignersupport::testItemReparenting()
 
     QCOMPARE(text->parentItem(), rootItem);
     QQmlProperty childrenProperty(rootItem, "children");
-    removeObjectFromList(childrenProperty, text, view->engine());
+    removeObjectFromList(childrenProperty, text);
     addToNewProperty(text, item, "children");
     QCOMPARE(text->parentItem(), item);
 }
