@@ -1,30 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the tools applications of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include <QtQml/qqmlengine.h>
 #include <QtQml/private/qqmlengine_p.h>
@@ -521,7 +496,7 @@ public:
             }
         }
 
-        for (const QMetaObject *meta : qAsConst(objectsToMerge)) {
+        for (const QMetaObject *meta : std::as_const(objectsToMerge)) {
             for (int index = meta->enumeratorOffset(); index < meta->enumeratorCount(); ++index)
                 dump(meta->enumerator(index));
 
@@ -654,11 +629,11 @@ private:
 
         if (typeName->endsWith('*')) {
             *isPointer = true;
-            typeName->truncate(typeName->length() - 1);
+            typeName->truncate(typeName->size() - 1);
             removePointerAndList(typeName, isList, isPointer);
         } else if (typeName->startsWith(declListPrefix)) {
             *isList = true;
-            typeName->truncate(typeName->length() - 1); // get rid of the suffix '>'
+            typeName->truncate(typeName->size() - 1); // get rid of the suffix '>'
             *typeName = typeName->mid(declListPrefix.size());
             removePointerAndList(typeName, isList, isPointer);
         }
@@ -835,7 +810,7 @@ static bool readDependenciesData(QString dependenciesFile, const QByteArray &fil
                     continue;
                 if (name.contains(QLatin1String("Private"), Qt::CaseInsensitive)) {
                     if (verbose)
-                        std::cerr << "skipping private dependecy "
+                        std::cerr << "skipping private dependency "
                                   << qPrintable( name ) << " "  << qPrintable(version) << std::endl;
                     continue;
                 }
@@ -910,7 +885,7 @@ static bool getDependencies(const QQmlEngine &engine, const QString &pluginImpor
 
     if (!importScanner.waitForFinished()) {
         std::cerr << "failure to start " << qPrintable(command);
-        for (const QString &arg : qAsConst(commandArgs))
+        for (const QString &arg : std::as_const(commandArgs))
             std::cerr << ' ' << qPrintable(arg);
         std::cerr << std::endl;
         return false;
@@ -945,18 +920,18 @@ bool dependencyBetter(const QString &lhs, const QString &rhs)
     if (leftModule > rightModule)
         return false;
 
-    if (leftSegments.length() == 1)
+    if (leftSegments.size() == 1)
         return false;
-    if (rightSegments.length() == 1)
+    if (rightSegments.size() == 1)
         return true;
 
     const QStringList leftVersion = leftSegments.at(1).split(QLatin1Char('.'));
     const QStringList rightVersion = rightSegments.at(1).split(QLatin1Char('.'));
 
     auto compareSegment = [&](int segmentIndex) {
-        if (leftVersion.length() <= segmentIndex)
-            return rightVersion.length() > segmentIndex ? 1 : 0;
-        if (rightVersion.length() <= segmentIndex)
+        if (leftVersion.size() <= segmentIndex)
+            return rightVersion.size() > segmentIndex ? 1 : 0;
+        if (rightVersion.size() <= segmentIndex)
             return -1;
 
         bool leftOk = false;
@@ -1223,7 +1198,7 @@ int main(int argc, char *argv[])
     // load the QtQml builtins and the dependencies
     {
         QByteArray code(qtQmlImportString.toUtf8());
-        for (const QString &moduleToImport : qAsConst(dependencies)) {
+        for (const QString &moduleToImport : std::as_const(dependencies)) {
             code.append("\nimport ");
             code.append(moduleToImport.toUtf8());
         }
@@ -1287,7 +1262,7 @@ int main(int argc, char *argv[])
                         QString::number(qtObjectType.version().minorVersion())).toUtf8();
         }
         // avoid importing dependencies?
-        for (const QString &moduleToImport : qAsConst(dependencies)) {
+        for (const QString &moduleToImport : std::as_const(dependencies)) {
             importCode.append("\nimport ");
             importCode.append(moduleToImport.toUtf8());
         }
@@ -1324,7 +1299,7 @@ int main(int argc, char *argv[])
             it->erase(std::unique(it->begin(), it->end()), it->end());
         }
 
-        for (const QMetaObject *mo : qAsConst(candidates)) {
+        for (const QMetaObject *mo : std::as_const(candidates)) {
             if (mo->className() != QLatin1String("Qt"))
                 metas.insert(mo);
         }
@@ -1352,13 +1327,13 @@ int main(int argc, char *argv[])
 
     // put the metaobjects into a map so they are always dumped in the same order
     QMap<QString, const QMetaObject *> nameToMeta;
-    for (const QMetaObject *meta : qAsConst(metas))
+    for (const QMetaObject *meta : std::as_const(metas))
         nameToMeta.insert(convertToId(meta), meta);
 
     Dumper dumper(&qml);
     if (relocatable)
         dumper.setRelocatableModuleUri(pluginImportUri);
-    for (const QMetaObject *meta : qAsConst(nameToMeta)) {
+    for (const QMetaObject *meta : std::as_const(nameToMeta)) {
         dumper.dump(QQmlEnginePrivate::get(&engine), meta, uncreatableMetas.contains(meta), singletonMetas.contains(meta));
     }
 
